-- *******************************************************
--   KOHA  MARC 21 STANDARD DEFAULT AUTHORITY FRAMEWORKS
--
--        Revised to Update No. 21 (September 2015)
-- *******************************************************

-- ******************************************************
-- KOHA DEFAULT MARC 21 AUTHORITY TYPE.
-- ******************************************************

-- This authority type includes all fields and subfieds used by any
-- authority type.

INSERT INTO auth_types (authtypecode, authtypetext, auth_tag_to_report, summary) VALUES
		('', 'Default', '', '');


-- ******************************************************
-- KOHA SUPPORTED STANDARD MARC 21 AUTHORITY TYPES.
-- ******************************************************

-- These authority types are supported for for guiding the cataloguer to
-- fill authorised values in Koha MARC bibliographic editor.


INSERT INTO auth_types (authtypecode, authtypetext, auth_tag_to_report, summary) VALUES
		('PERSO_NAME', 'Nome di persona', '100', 'Nome di persona'),
		('CORPO_NAME', 'Nome di ente', '110', 'Nome di ente'),
		('MEETI_NAME', 'Nome di convegno', '111', 'Nome di convegno'),
		('UNIF_TITLE', 'Titolo uniforme', '130', 'Titolo uniforme'),
		('CHRON_TERM', 'Termine cronologico', '148', 'Termine cronologico'),
		('TOPIC_TERM', 'Termine topico', '150', 'Termine topico'),
		('GEOGR_NAME', 'Nome geografico', '151', 'Nome geografico'),
		('GENRE/FORM', 'Termine di genere/forma', '155', 'Termine di genere/forma');


-- ******************************************************
-- KOHA UNSUPPORTED STANDARD MARC 21 AUTHORITY TYPES.
-- ******************************************************

-- These authority types are only supported for guiding the cataloguer to
-- fill authorised values in the Koha MARC bibliographic editor to the
-- extent that they have already been included in a primary authority type,
-- therefore, they have not yet been specified.
--
-- Minimal primary authorities including subdivisions may currently be
-- built by a script which uses the values in bibliographic records but
-- include no tracings and references which are necessarily not present
-- in bibliographic records.


-- INSERT INTO `auth_types` (`authtypecode`, `authtypetext`, `auth_tag_to_report`, `summary`) VALUES
--		('TOPIC_SUBD', 'General Topical Term Subdivision', '180', 'General Topical Term Subdivision'),
--		('GEOGR_SUBD', 'Geographic Subdivision', '181', 'Geographic Subdivision'),
--		('CHRON_SUBD', 'Chronological Subdivision', '182', 'Chronological Subdivision'),
--		('FORM_SUBDI', 'Form Subdivision', '185', 'Form Subdivision');



-- *****************************************************************
-- MARC 21 AUTHORITY FIELDS/SUBFIELDS AND COMMMONLY USED EXTENSIONS
-- *****************************************************************


-- No support for subfield visibility is yet provided by the Koha code
-- matching the column name 'hidden' in the bibliographic framework.
--
-- Needs checking for errors but probably tolerable for use on a production.
-- A server can be upgraded easily from later versions of this file.
--
-- In the absense of more column support for qualifying the relative
-- importance of subfields to the record editor, some modest modification of
-- the default framework is needed setting the not-useful non-Koha holdings
-- subfields to not managed in Koha.

-- MARC fields including letters as part of the field identifier are from RLIN
-- and should be expected to remain along with RLIN $% subfields.  RLIN has
-- been using letters in fields because there are not enough local use number
-- fields which have not already been specified for very large union catalogue
-- networks such as RLIN itself.

-- Fields ending in a or b are temporary placeholders for information from
-- a numeric value until a non-conflicting way to treat the content under the
-- proper original numeric field is adopted.  090 for LC call numbers is much
-- too common and important so 999 is also provided as a temporary place
-- holder until all Koha code for finding control fields has been changed from
-- a numeric test of < 10 to a regular expression match of m/^00/ to prevent
-- mistaken matching of fields with letters such as 09o if they were control
-- fields.

-- Fields and subfields which belong exclusively in a classification
-- framework have been marked '[CLASSIFICATION FRAMEWORK]'.  They are retained
-- or included here becuase some had been included previously and Koha does
-- not yet have a classification framework.


INSERT INTO `auth_tag_structure` (`authtypecode`, `tagfield`, `liblibrarian`, `libopac`, `repeatable`, `mandatory`, `authorised_value`) VALUES
		('', '000', 'LEADER', 'LEADER', 0, 1, NULL),
		('', '001', 'Numero di controllo', 'Numero di controllo', 0, 1, NULL),
		('', '003', 'Identificatore del numero di controllo', 'Identificatore del numero di controllo', 0, 1, NULL),
		('', '005', 'Data e orario dell\'ultima transazione', 'Data e orario dell\'ultima transazione', 0, 1, NULL),
		('', '008', 'Elementi a lunghezza fissa', 'Elementi a lunghezza fissa', 0, 1, NULL),
		('', '010', 'Numero di controllo della Library of Congress', 'Numero di controllo della Library of Congress', 0, 0, NULL),
		('', '014', 'Collegamento alla registrazione bibliografica di seriale o di item in più parti', 'Collegamento alla registrazione bibliografica di seriale o di item in più parti', 1, 0, NULL),
		('', '016', 'Numero di controllo dell\'agenzia bibliografica nazionale', 'Numero di controllo dell\'agenzia bibliografica nazionale', 1, 0, NULL),
		('', '020', 'ISBN (International Standard Book Number)', 'ISBN (International Standard Book Number)', 1, 0, NULL),
		('', '022', 'ISSN (International Standard Serial Number)', 'ISSN (International Standard Serial Number)', 1, 0, NULL),
		('', '024', 'Altri identificatori standard', 'Altri identificatori standard', 1, 0, NULL),
		('', '031', 'Informazione sugli incipit musicali', 'Informazione sugli incipit musicali', 1, 0, NULL),
		('', '034', 'Dati matematici codificati per il materiale cartografico', 'Dati matematici codificati per il materiale cartografico', 1, 0, NULL),
		('', '035', 'Numero di controllo del sistema', 'Numero di controllo del sistema', 1, 0, NULL),
		('', '040', 'Fonte della catalogazione', 'Fonte della catalogazione', 0, 1, NULL),
		('', '042', 'Codice d\'autenticazione', 'Codice d\'autenticazione', 0, 0, NULL),
		('', '043', 'Codice dell\'area geografica', 'Codice dell\'area geografica', 0, 0, NULL),
		('', '045', 'Periodo di tempo del contenuto', 'Periodo di tempo del contenuto', 0, 0, NULL),
		('', '046', 'Codice delle date speciali', 'Codice delle date speciali', 1, 0, NULL),
		('', '050', 'Numero di chiamata (collocazione) della Library of Congress', 'Numero di chiamata (collocazione) della Library of Congress', 1, 0, NULL),
		('', '052', 'Classificazione geografica', 'Classificazione geografica', 1, 0, NULL),
		('', '053', 'Numero di classificazione della LC', 'Numero di classificazione della LC', 1, 0, NULL),
		('', '055', 'Numero di chiamata (collocazione) della Library and Archives Canada', 'Numero di chiamata (collocazione) della Library and Archives Canada', 1, 0, NULL),
		('', '058', 'LC CLASSIFICATION NUMBER ASSIGNED IN CANADA [OBSOLETE, CAN/MARC]', 'LC CLASSIFICATION NUMBER ASSIGNED IN CANADA [OBSOLETE, CAN/MARC]', 1, 0, NULL),
		('', '060', 'Numero di chiamata (collocazione) della National Library of Medicine', 'Numero di chiamata (collocazione) della National Library of Medicine', 1, 0, NULL),
		('', '063', 'NLM CLASSIFICATION NUMBER ASSIGNED BY NLM [OBSOLETE, CAN/MARC]', 'NLM CLASSIFICATION NUMBER ASSIGNED BY NLM [OBSOLETE, CAN/MARC]', 1, 0, NULL),
		('', '065', 'Numero d’altra classificazione', 'Numero d’altra classificazione', 1, 0, NULL),
		('', '066', 'Set di caratteri presenti', 'Set di caratteri presenti', 0, 0, NULL),
		('', '068', 'NLM CLASSIFICATION NUMBER ASSIGNED IN CANADA [OBSOLETE, CAN/MARC]', 'NLM CLASSIFICATION NUMBER ASSIGNED IN CANADA [OBSOLETE, CAN/MARC]', 1, 0, NULL),
		('', '070', 'Numero di chiamata (collocazione) della National Agricultural Library', 'Numero di chiamata (collocazione) della National Agricultural Library', 1, 0, NULL),
		('', '072', 'Codice di categoria di soggetto', 'Codice di categoria di soggetto', 1, 0, NULL),
		('', '073', 'Impiego della suddivisione', 'Impiego della suddivisione', 0, 0, NULL),
		('', '080', 'Numero di classificazione decimale universale', 'Numero di classificazione decimale universale', 1, 0, NULL),
		('', '082', 'Numero di chiamata (collocazione) della decimale Dewey', 'Numero di chiamata (collocazione) della decimale Dewey', 1, 0, NULL),
		('', '083', 'Numero di classificazione Decimale Dewey', 'Numero di classificazione Decimale Dewey', 1, 0, NULL),
		('', '084', 'Schema di classificazione e edizione [Formato per classificazione]', 'Schema di classificazione e edizione [Formato per classificazione]', 0, 0, NULL),
		('', '086', 'Numero di chiamata (collocazione) di documento governativo', 'Numero di chiamata (collocazione) di documento governativo', 1, 0, NULL),
		('', '087', 'Numero di classificazione di documento governativo', 'Numero di classificazione di documento governativo', 1, 0, NULL),
		('', '088', 'DOCUMENT SHELVING NUMBER (CODOC) [OBSOLETE, CAN/MARC]', 'DOCUMENT SHELVING NUMBER (CODOC) [OBSOLETE, CAN/MARC]', 1, 0, NULL),
		('', '090', 'LOCAL CALL NUMBER (SERIES) [OBSOLETE]; LOCALLY ASSIGNED LC-TYPE CALL NUMBER (OCLC); LOCAL CALL NUMBER (RLIN)', 'LOCAL CALL NUMBER (SERIES) [OBSOLETE]; LOCALLY ASSIGNED LC-TYPE CALL NUMBER (OCLC); LOCAL CALL NUMBER (RLIN)', 1, 0, NULL),
		('', '091', 'LOCALLY ASSIGNED LC-TYPE CLASSIFICATION NUMBER (OCLC); LOCAL CLASSIFICATION NUMBER (RLIN)', 'LOCALLY ASSIGNED LC-TYPE CLASSIFICATION NUMBER (OCLC); LOCAL CLASSIFICATION NUMBER (RLIN)', 0, 0, NULL),
		('', '092', 'LOCALLY ASSIGNED DEWEY CALL NUMBER (OCLC)', 'LOCALLY ASSIGNED DEWEY CALL NUMBER (OCLC)', 1, 0, NULL),
		('', '093', 'LOCALLY ASSIGNED DEWEY CLASSIFICATION NUMBER (OCLC)', 'LOCALLY ASSIGNED DEWEY CLASSIFICATION NUMBER (OCLC)', 1, 0, NULL),
		('', '096', 'LOCALLY ASSIGNED NLM-TYPE CALL NUMBER (OCLC)', 'LOCALLY ASSIGNED NLM-TYPE CALL NUMBER (OCLC)', 1, 0, NULL),
		('', '097', 'LOCALLY ASSIGNED NLM-TYPE CLASSIFICATION NUMBER (OCLC)', 'LOCALLY ASSIGNED NLM-TYPE CLASSIFICATION NUMBER (OCLC)', 1, 0, NULL),
		('', '098', 'OTHER CLASSIFICATION SCHEMES (OCLC)', 'OTHER CLASSIFICATION SCHEMES (OCLC)', 1, 0, NULL),
		('', '099', 'LOCAL FREE-TEXT CALL NUMBER (OCLC)', 'LOCAL FREE-TEXT CALL NUMBER (OCLC)', 1, 0, NULL),
		('', '100', 'Intestazione-Nome di persona', 'Intestazione-Nome di persona', 0, 0, NULL),
		('', '110', 'Intestazione-Nome di ente', 'Intestazione-Nome di ente', 0, 0, NULL),
		('', '111', 'Intestazione-Nome di convegno', 'Intestazione-Nome di convegno', 0, 0, NULL),
		('', '130', 'Intestazione-Titolo uniforme', 'Intestazione-Titolo uniforme', 0, 0, NULL),
		('', '140', 'UNIFORM TITLE [OBSOLETE, CAN/MARC]', 'UNIFORM TITLE [OBSOLETE, CAN/MARC]', 1, 0, NULL),
		('', '143', 'COLLECTIVE TITLE [OBSOLETE, CAN/MARC]', 'COLLECTIVE TITLE [OBSOLETE, CAN/MARC]', 1, 0, NULL),
		('', '148', 'Intestazione-Termine cronologico', 'Intestazione-Termine cronologico', 0, 0, NULL),
		('', '150', 'Intestazione-Termine topico', 'Intestazione-Termine topico', 0, 0, NULL),
		('', '151', 'Intestazione – Nome geografico', 'Intestazione – Nome geografico', 0, 0, NULL),
		('', '153', 'Numero classificazione [Formato per classificazione]]', 'Numero classificazione [Formato per classificazione]]', 0, 0, NULL),
		('', '154', 'Termine d\'indice, spiegazione generale [Formato per classificazione]', 'Termine d\'indice, spiegazione generale [Formato per classificazione]', 0, 0, NULL),
		('', '155', 'Intestazione-Termine di genere/forma', 'Intestazione-Termine di genere/forma', 0, 0, NULL),
		('', '162', 'Intestazione – Termine del mezzo di esecuzione', 'Intestazione – Termine del mezzo di esecuzione', 0, 0, NULL),
		('', '180', 'Intestazione-Suddivisione generale', 'Intestazione-Suddivisione generale', 0, 0, NULL),
		('', '181', 'Intestazione-Suddivisione geografica', 'Intestazione-Suddivisione geografica', 0, 0, NULL),
		('', '182', 'Intestazione-Suddivisione cronologica', 'Intestazione-Suddivisione cronologica', 0, 0, NULL),
		('', '185', 'Intestazione-Suddivisione formale', 'Intestazione-Suddivisione formale', 0, 0, NULL),
		('', '253', 'Riferimento \'vedi\' complesso [Formato per classificazione]', 'Riferimento \'vedi\' complesso [Formato per classificazione]', 1, 0, NULL),
		('', '260', 'Rinvio complesso “Vedi”-Soggetto', 'Rinvio complesso “Vedi”-Soggetto', 1, 0, NULL),
		('', '336', 'Tipo contenuto', 'Tipo contenuto', 1, 0, NULL),
		('', '348', 'Presentazione musicale', 'Presentazione musicale', 1, 0, NULL),
		('', '353', 'Riferimento \'vedi anche\' complesso [Formato per classificazione]', 'Riferimento \'vedi anche\' complesso [Formato per classificazione]', 1, 0, NULL),
		('', '360', 'Rinvio complesso “Vedi anche”- Soggetto', 'Rinvio complesso “Vedi anche”- Soggetto', 1, 0, NULL),
		('', '368', 'Altri attributi della persona o dell\'ente', 'Altri attributi della persona o dell\'ente', 1, 0, NULL),
		('', '370', 'Luogo associato', 'Luogo associato', 1, 0, NULL),
		('', '371', 'Indirizzo', 'Indirizzo', 1, 0, NULL),
		('', '372', 'Campo di attività', 'Campo di attività', 1, 0, NULL),
		('', '373', 'Affiliazione', 'Affiliazione', 1, 0, NULL),
		('', '374', 'Occupazione', 'Occupazione', 1, 0, NULL),
		('', '375', 'Sesso', 'Sesso', 1, 0, NULL),
		('', '376', 'Informazione sulla famiglia', 'Informazione sulla famiglia', 1, 0, NULL),
		('', '377', 'Lingua associata', 'Lingua associata', 1, 0, NULL),
		('', '378', 'Nome di persona - forma espansa', 'Nome di persona - forma espansa', 0, 0, NULL),
		('', '380', 'Forma dell\'emissione', 'Forma dell\'emissione', 1, 0, NULL),
		('', '381', 'Altre caratteristiche distintive dell\'emissione o espressione', 'Altre caratteristiche distintive dell\'emissione o espressione', 1, 0, NULL),
		('', '382', 'Modalità dell\'esecuzione', 'Modalità dell\'esecuzione', 1, 0, NULL),
		('', '383', 'Designazione numerica dell\'opera musicale', 'Designazione numerica dell\'opera musicale', 1, 0, NULL),
		('', '384', 'Chiave musicale', 'Chiave musicale', 0, 0, NULL),
		('', '385', 'Caratteristiche del pubblico', 'Caratteristiche del pubblico', 1, 0, NULL),
		('', '386', 'Caratteristiche del creatore/contributore', 'Caratteristiche del creatore/contributore', 1, 0, NULL),
		('', '388', 'Periodo di tempo della creazione del contenuto', 'Periodo di tempo della creazione del contenuto', 1, 0, NULL),
		('', '400', 'Tracciato “Vedi”- Nome di persona', 'Tracciato “Vedi”- Nome di persona', 1, 0, NULL),
		('', '410', 'Tracciato “Vedi”-Nome di ente', 'Tracciato “Vedi”-Nome di ente', 1, 0, NULL),
		('', '411', 'Tracciato “Vedi”-Nome di convegno', 'Tracciato “Vedi”-Nome di convegno', 1, 0, NULL),
		('', '430', 'Tracciato “Vedi”-Titolo uniforme', 'Tracciato “Vedi”-Titolo uniforme', 1, 0, NULL),
		('', '448', 'Tracciato “Vedi”-Termine cronologico', 'Tracciato “Vedi”-Termine cronologico', 1, 0, NULL),
		('', '450', 'Tracciato “Vedi”-Termine topico', 'Tracciato “Vedi”-Termine topico', 1, 0, NULL),
		('', '451', 'Tracciato “Vedi”-Nome geografico', 'Tracciato “Vedi”-Nome geografico', 1, 0, NULL),
		('', '453', 'Tracciato di un numero invalido [Formato per classificazione]', 'Tracciato di un numero invalido [Formato per classificazione]', 1, 0, NULL),
		('', '455', 'Tracciato “Vedi”-Termine di genere/forma', 'Tracciato “Vedi”-Termine di genere/forma', 1, 0, NULL),
		('', '462', 'Tracciato “Vedi”- Termine del mezzo di esecuzione', 'Tracciato “Vedi”- Termine del mezzo di esecuzione', 1, 0, NULL),
		('', '480', 'Tracciato “Vedi”-Suddivisione generale', 'Tracciato “Vedi”-Suddivisione generale', 1, 0, NULL),
		('', '481', 'Tracciato “Vedi”-Suddivisione geografica', 'Tracciato “Vedi”-Suddivisione geografica', 1, 0, NULL),
		('', '482', 'Tracciato “Vedi”-Suddivisione cronologica', 'Tracciato “Vedi”-Suddivisione cronologica', 1, 0, NULL),
		('', '485', 'Tracciato “Vedi”-Suddivisione formale', 'Tracciato “Vedi”-Suddivisione formale', 1, 0, NULL),
		('', '500', 'Tracciato “Vedi anche”-Nome di persona', 'Tracciato “Vedi anche”-Nome di persona', 1, 0, NULL),
		('', '510', 'Tracciato “Vedi anche”-Nome di ente', 'Tracciato “Vedi anche”-Nome di ente', 1, 0, NULL),
		('', '511', 'Tracciato “Vedi anche”-Nome di convegno', 'Tracciato “Vedi anche”-Nome di convegno', 1, 0, NULL),
		('', '530', 'Tracciato “Vedi anche”-Titolo uniforme', 'Tracciato “Vedi anche”-Titolo uniforme', 1, 0, NULL),
		('', '548', 'Tracciato “Vedi anche”-Termine cronologico', 'Tracciato “Vedi anche”-Termine cronologico', 1, 0, NULL),
		('', '550', 'Tracciato “Vedi anche”-Termine topico', 'Tracciato “Vedi anche”-Termine topico', 1, 0, NULL),
		('', '551', 'Tracciato “Vedi anche”-Nome geografico', 'Tracciato “Vedi anche”-Nome geografico', 1, 0, NULL),
		('', '553', 'Tracciato di un numero valido [Formato per classificazione]', 'Tracciato di un numero valido [Formato per classificazione]', 1, 0, NULL),
		('', '555', 'Tracciato “Vedi anche”-Termine di genere/forma', 'Tracciato “Vedi anche”-Termine di genere/forma', 1, 0, NULL),
		('', '562', 'Tracciato “Vedi anche ”- Termine del mezzo di esecuzione', 'Tracciato “Vedi anche ”- Termine del mezzo di esecuzione', 1, 0, NULL),
		('', '580', 'Tracciato “Vedi anche”-Suddivisione generale', 'Tracciato “Vedi anche”-Suddivisione generale', 1, 0, NULL),
		('', '581', 'Tracciato “Vedi anche”-Suddivisione geografica', 'Tracciato “Vedi anche”-Suddivisione geografica', 1, 0, NULL),
		('', '582', 'Tracciato “Vedi anche”-Suddivisione cronologica', 'Tracciato “Vedi anche”-Suddivisione cronologica', 1, 0, NULL),
		('', '585', 'Tracciato “Vedi anche”-Suddivisione formale', 'Tracciato “Vedi anche”-Suddivisione formale', 1, 0, NULL),
		('', '640', 'Date di pubblicazione e/o designazione sequenziale della serie', 'Date di pubblicazione e/o designazione sequenziale della serie', 1, 0, NULL),
		('', '641', 'Particolarità della numerazione della serie', 'Particolarità della numerazione della serie', 1, 0, NULL),
		('', '642', 'Esempio di numerazione della serie', 'Esempio di numerazione della serie', 1, 0, NULL),
		('', '643', 'Luogo e editore/ente responsabile della serie', 'Luogo e editore/ente responsabile della serie', 1, 0, NULL),
		('', '644', 'Procedura di analisi della serie', 'Procedura di analisi della serie', 1, 0, NULL),
		('', '645', 'Procedura di tracciato della serie', 'Procedura di tracciato della serie', 1, 0, NULL),
		('', '646', 'Procedura di classificazione della serie', 'Procedura di classificazione della serie', 1, 0, NULL),
		('', '663', 'Rinvio complesso “Vedi anche”-Nome', 'Rinvio complesso “Vedi anche”-Nome', 0, 0, NULL),
		('', '664', 'Rinvio complesso “Vedi”-Nome', 'Rinvio complesso “Vedi”-Nome', 0, 0, NULL),
		('', '665', 'Rinvio storico', 'Rinvio storico', 0, 0, NULL),
		('', '666', 'Rinvio esplicativo generale-Nome', 'Rinvio esplicativo generale-Nome', 0, 0, NULL),
		('', '667', 'Nota generale non pubblica', 'Nota generale non pubblica', 1, 0, NULL),
		('', '668', 'CHARACTERS IN NONROMAN ALPHABETS [OBSOLETE]', 'CHARACTERS IN NONROMAN ALPHABETS [OBSOLETE]', 1, 0, NULL),
		('', '670', 'Fonte dei dati trovati', 'Fonte dei dati trovati', 1, 0, NULL),
		('', '671', 'NOTE--WORK CATALOGUED [OBSOLETE, CAN/MARC ONLY]', 'NOTE--WORK CATALOGUED [OBSOLETE, CAN/MARC ONLY]', 1, 0, NULL),
		('', '672', 'Titolo collegato all\'entità', 'Titolo collegato all\'entità', 1, 0, NULL),
		('', '673', 'Titolo non collegato all\'entità', 'Titolo non collegato all\'entità', 1, 0, NULL),
		('', '675', 'Fonte dei dati non trovati', 'Fonte dei dati non trovati', 0, 0, NULL),
		('', '676', 'NOTE--CATALOGUING RULES (NAMES/TITLES) [OBSOLETE, CAN/MARC ONLY]', 'NOTE--CATALOGUING RULES (NAMES/TITLES) [OBSOLETE, CAN/MARC ONLY]', 0, 0, NULL),
		('', '678', 'Dati biografici o storici', 'Dati biografici o storici', 1, 0, NULL),
		('', '680', 'Nota generale pubblica', 'Nota generale pubblica', 1, 0, NULL),
		('', '681', 'Nota di tracciato per esempio di soggetto', 'Nota di tracciato per esempio di soggetto', 1, 0, NULL),
		('', '682', 'Informazione sull\'intestazione cancellata', 'Informazione sull\'intestazione cancellata', 0, 0, NULL),
		('', '683', 'Nota sulla storia dell\'applicazione', 'Nota sulla storia dell\'applicazione', 1, 0, NULL),
		('', '684', 'Nota istruzione ausiliaria [Formato per classificazione]', 'Nota istruzione ausiliaria [Formato per classificazione]', 1, 0, NULL),
		('', '685', 'NOTE--SOURCE DATA FOUND (SUBJECTS) [OBSOLETE, CAN/MARC ONLY]', 'NOTE--SOURCE DATA FOUND (SUBJECTS) [OBSOLETE, CAN/MARC ONLY]', 1, 0, NULL),
		('', '686', 'NOTE--SOURCE DATA NOT FOUND (SUBJECTS) [OBSOLETE, CAN/MARC ONLY]', 'NOTE--SOURCE DATA NOT FOUND (SUBJECTS) [OBSOLETE, CAN/MARC ONLY]', 1, 0, NULL),
		('', '687', 'NOTE--USAGE (SUBJECTS) [OBSOLETE, CAN/MARC ONLY]', 'NOTE--USAGE (SUBJECTS) [OBSOLETE, CAN/MARC ONLY]', 1, 0, NULL),
		('', '688', 'Nota sulla storia dell\'applicazione', 'Nota sulla storia dell\'applicazione', 1, 0, NULL),
		('', '700', 'Collegamento di intestazione stabilita-Nome di persona', 'Collegamento di intestazione stabilita-Nome di persona', 1, 0, NULL),
		('', '710', 'Collegamento di intestazione stabilita-Nome di ente', 'Collegamento di intestazione stabilita-Nome di ente', 1, 0, NULL),
		('', '711', 'Collegamento di intestazione stabilita-Nome di convegno', 'Collegamento di intestazione stabilita-Nome di convegno', 1, 0, NULL),
		('', '730', 'Collegamento di intestazione stabilita-Titolo uniforme', 'Collegamento di intestazione stabilita-Titolo uniforme', 1, 0, NULL),
		('', '748', 'Collegamento di intestazione stabilita-Termine cronologico', 'Collegamento di intestazione stabilita-Termine cronologico', 1, 0, NULL),
		('', '750', 'Collegamento di intestazione stabilita-Termine topico (R)', 'Collegamento di intestazione stabilita-Termine topico (R)', 1, 0, NULL),
		('', '751', 'Collegamento di intestazione stabilita-Nome geografico', 'Collegamento di intestazione stabilita-Nome geografico', 1, 0, NULL),
		('', '753', 'Termine d\'indice non controllato [Formato per classificazione]', 'Termine d\'indice non controllato [Formato per classificazione]', 1, 0, NULL),
		('', '754', 'Termine d\'indice -- termini a faccette [Formato per classificazione]', 'Termine d\'indice -- termini a faccette [Formato per classificazione]', 1, 0, NULL),
		('', '755', 'Collegamento di intestazione stabilita-Termine di genere/forma', 'Collegamento di intestazione stabilita-Termine di genere/forma', 1, 0, NULL),
		('', '761', 'Instruzioni per aggiungere o dividere [Formato per classificazione]', 'Instruzioni per aggiungere o dividere [Formato per classificazione]', 1, 0, NULL),
		('', '762', 'Collegamento di intestazione stabilita-Termine del mezzo di esecuzione', 'Collegamento di intestazione stabilita-Termine del mezzo di esecuzione', 1, 0, NULL),
		('', '763', 'Codice in una sottosezione o tabella [Formato per classificazione]', 'Codice in una sottosezione o tabella [Formato per classificazione]', 1, 0, NULL),
		('', '764', 'Identificazione della regola [Formato per classificazione]', 'Identificazione della regola [Formato per classificazione]', 1, 0, NULL),
		('', '765', 'Componenti numero sintetizzato [Formato per classificazione]', 'Componenti numero sintetizzato [Formato per classificazione]', 1, 0, NULL),
		('', '766', 'Tabella di informazione secondaria [Formato per classificazione]', 'Tabella di informazione secondaria [Formato per classificazione]', 1, 0, NULL),
		('', '768', 'Istruzioni per citazione e ordine di preferenza [Formato per classificazione]', 'Istruzioni per citazione e ordine di preferenza [Formato per classificazione]', 1, 0, NULL),
		('', '780', 'Collegamento di suddivisione-Suddivisione generale', 'Collegamento di suddivisione-Suddivisione generale', 1, 0, NULL),
		('', '781', 'Collegamento di suddivisione-Suddivisione geografica', 'Collegamento di suddivisione-Suddivisione geografica', 1, 0, NULL),
		('', '782', 'Collegamento di suddivisione-Suddivisione cronologica', 'Collegamento di suddivisione-Suddivisione cronologica', 1, 0, NULL),
		('', '785', 'Collegamento di suddivisione-Suddivisione formale', 'Collegamento di suddivisione-Suddivisione formale', 1, 0, NULL),
		('', '788', 'Dati di collegamento complesso', 'Dati di collegamento complesso', 0, 0, NULL),
		('', '856', 'Localizzazione e accesso elettronico', 'Localizzazione e accesso elettronico', 1, 0, NULL),
		('', '880', 'Rappresentazione grafica alternativa', 'Rappresentazione grafica alternativa', 1, 0, NULL),
		('', '883', 'Provenienza dei metadati generati dalla macchina', 'Provenienza dei metadati generati dalla macchina', 1, 0, NULL),
		('', '884', 'Descrizione della conversione', 'Descrizione della conversione', 1, 0, NULL),
		('', '942', 'Koha, uso interno', 'Koha, uso interno', 0, 1, NULL);



INSERT INTO `auth_subfield_structure` (`authtypecode`, `tagfield`, `tagsubfield`, `liblibrarian`, `libopac`, `repeatable`, `mandatory`, `tab`, `authorised_value`, `value_builder`, `seealso`, `isurl`, `hidden`, `linkid`, `kohafield`, `frameworkcode`) VALUES
		('', '000', '@', 'campo di controllo lung. fissa', 'campo di controllo lung. fissa', 0, 1, 0, NULL, 'marc21_leader_authorities.pl', NULL, 0, 0, '', '', ''),
		('', '001', '@', 'campo di controllo', 'campo di controllo', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', 'auth_header.authid', ''),
		('', '003', '@', 'campo di controllo', 'campo di controllo', 0, 1, 0, NULL, 'marc21_field_003.pl', NULL, 0, 0, '', '', ''),
		('', '005', '@', 'campo di controllo', 'campo di controllo', 0, 1, 0, NULL, 'marc21_field_005.pl', NULL, 0, 0, '', '', ''),
		('', '008', '@', 'campo di controllo lung. fissa', 'campo di controllo lung. fissa', 0, 1, 0, NULL, 'marc21_field_008_authorities.pl', NULL, 0, 0, '', '', ''),
		('', '010', 'a', 'Numero di controllo della LC', 'Numero di controllo della LC', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '010', 'z', 'Numero di controllo della LC cancellato o errato', 'Numero di controllo della LC cancellato o errato', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '010', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '014', 'a', 'Nunero di controllo del record bibliografico collegato', 'Nunero di controllo del record bibliografico collegato', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '014', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '014', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '016', 'a', 'Numero di controllo della registrazione', 'Numero di controllo della registrazione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '016', 'z', 'Numero di controllo della registrazione cancellato o errato', 'Numero di controllo della registrazione cancellato o errato', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '016', '2', 'Fonte', 'Fonte', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '016', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '020', 'a', 'ISBN (International Standard Book Number)', 'ISBN (International Standard Book Number)', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '020', 'c', 'Termini di disponibilità', 'Termini di disponibilità', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '020', 'q', 'Qualificazione', 'Qualificazione', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '020', 'z', 'ISBN cancellato o errato', 'ISBN cancellato o errato', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '020', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '020', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '022', 'a', 'ISSN (International Standard Serial Number)', 'ISSN (International Standard Serial Number)', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '022', 'l', 'ISSN-L', 'ISSN-L', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '022', 'm', 'ISSN-L cancellato', 'ISSN-L cancellato', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '022', 'y', 'ISSN errato', 'ISSN errato', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '022', 'z', 'ISSN cancellato', 'ISSN cancellato', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '022', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '022', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '024', 'a', 'Numero standard o codice', 'Numero standard o codice', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '024', 'c', 'Termini di disponibilità', 'Termini di disponibilità', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '024', 'd', 'Codici addizionali che seguono il numero standard o codice', 'Codici addizionali che seguono il numero standard o codice', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '024', 'q', 'Qualificazione', 'Qualificazione', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '024', 'z', 'Numero standard o codice, cancellato o errato', 'Numero standard o codice, cancellato o errato', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '024', '2', 'Fonte del numero o codice', 'Fonte del numero o codice', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '024', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '024', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', 'a', 'Numero di composizione', 'Numero di composizione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', 'b', 'Numero di movimento', 'Numero di movimento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', 'c', 'Numero progressivo nel movimento', 'Numero progressivo nel movimento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', 'd', 'Titolo o intitolazione', 'Titolo o intitolazione', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', 'e', 'Personaggio', 'Personaggio', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', 'g', 'Chiave', 'Chiave', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', 'm', 'Voce/strumento', 'Voce/strumento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', 'n', 'Alterazioni', 'Alterazioni', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', 'o', 'Misura', 'Misura', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', 'p', 'Contesto musicale', 'Contesto musicale', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', 'q', 'Nota generale', 'Nota generale', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', 'r', 'Tonalità o modo', 'Tonalità o modo', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', 's', 'Nota codificata di validità', 'Nota codificata di validità', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', 't', 'Incipit testuale', 'Incipit testuale', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', 'u', 'URI (Uniform Resource Identifier)', 'URI (Uniform Resource Identifier)', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', 'y', 'Testo di collegamento', 'Testo di collegamento', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', 'z', 'Nota pubblica', 'Nota pubblica', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', '2', 'Codice di sistema', 'Codice di sistema', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '031', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', 'd', 'Coordinate--longitudine limite Ovest', 'Coordinate--longitudine limite Ovest', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', 'e', 'Coordinate--longitudine limite Est', 'Coordinate--longitudine limite Est', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', 'f', 'Coordinate--latitudine limite Nord', 'Coordinate--latitudine limite Nord', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', 'g', 'Coordinate--latitudine limite Sud', 'Coordinate--latitudine limite Sud', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', 'j', 'Declinazione--limite Nord', 'Declinazione--limite Nord', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', 'k', 'Declinazione--limite Sud', 'Declinazione--limite Sud', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', 'm', 'Ascensione retta--limite Est', 'Ascensione retta--limite Est', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', 'n', 'Ascensione retta--limite Ovest', 'Ascensione retta--limite Ovest', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', 'p', 'Equinozio', 'Equinozio', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', 'r', 'Coordinate--latitudine limite Sud', 'Coordinate--latitudine limite Sud', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', 's', 'Latitudine del perimetro', 'Latitudine del perimetro', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', 't', 'Longitudine del perimetro', 'Longitudine del perimetro', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', 'x', 'Data d\'inizio', 'Data d\'inizio', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', 'y', 'Data di fine', 'Data di fine', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', 'z', 'Nome dell\'oggetto extraterrestre', 'Nome dell\'oggetto extraterrestre', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', '0', 'Numero del record di authority o numero standard', 'Numero del record di authority o numero standard', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', '2', 'Fonte', 'Fonte', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', '3', 'Materiale specificato', 'Materiale specificato', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '034', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '035', 'a', 'System control number', 'System control number', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '035', 'z', 'Numero di controllo del sistema cancellato o errato', 'Numero di controllo del sistema cancellato o errato', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '035', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '035', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '040', 'a', 'Agenzia catalografica originale', 'Agenzia catalografica originale', 0, 1, 0, NULL, 'marc21_field_003.pl', NULL, 0, 0, '', '', ''),
		('', '040', 'b', 'Lingua della catalogazione', 'Lingua della catalogazione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '040', 'c', 'Agenzia che fa la trascrizione', 'Agenzia che fa la trascrizione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '040', 'd', 'Agenzia che fa la modifica', 'Agenzia che fa la modifica', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '040', 'e', 'Regole di descrizione', 'Regole di descrizione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '040', 'f', 'Convenzioni per le intestazioni di soggetto o tesauro', 'Convenzioni per le intestazioni di soggetto o tesauro', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '040', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '040', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '042', 'a', 'Codice d\'autenticazione', 'Codice d\'autenticazione', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '043', 'a', 'Codice dell\'area geografica', 'Codice dell\'area geografica', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '043', 'b', 'Codice locale GAC (codice dell\'area geografica)', 'Codice locale GAC (codice dell\'area geografica)', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '043', 'c', 'Codice ISO', 'Codice ISO', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '043', '0', 'Numero del record di authority o numero standard', 'Numero del record di authority o numero standard', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '043', '2', 'Fonte del codice di sottoente locale', 'Fonte del codice di sottoente locale', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '043', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '043', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '045', 'a', 'Codice di periodo di tempo', 'Codice di periodo di tempo', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '045', 'b', 'Periodo di tempo formattato dal 9999 a.C. fino al 9999 d.C.', 'Periodo di tempo formattato dal 9999 a.C. fino al 9999 d.C.', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '045', 'c', 'Periodo di tempo formattato anteriore al 9999 a.C.', 'Periodo di tempo formattato anteriore al 9999 a.C.', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '045', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '045', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '046', 'f', 'Data di nascita', 'Data di nascita', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '046', 'g', 'Data di morte', 'Data di morte', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '046', 'k', 'Data iniziale di creazione o data semplice', 'Data iniziale di creazione o data semplice', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '046', 'l', 'Data finale di creazione', 'Data finale di creazione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '046', 'o', 'Data singola o data iniziale per il contenuto aggregato', 'Data singola o data iniziale per il contenuto aggregato', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '046', 'p', 'Data finale per il contenuto aggregato', 'Data finale per il contenuto aggregato', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '046', 'q', 'Data di costituzione', 'Data di costituzione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '046', 'r', 'Data di cessazione', 'Data di cessazione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '046', 's', 'Inizio del periodo', 'Inizio del periodo', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '046', 't', 'Fine del periodo', 'Fine del periodo', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '046', 'u', 'URI (Uniform Resource Identifier)', 'URI (Uniform Resource Identifier)', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '046', 'v', 'Fonte dell\'informazione', 'Fonte dell\'informazione', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '046', '2', 'Fonte della data', 'Fonte della data', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '046', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '046', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '050', 'a', 'Numero di classificazione', 'Numero di classificazione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '050', 'b', 'Numero d\'item', 'Numero d\'item', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '050', 'd', 'Volumi/date a cui si applica il numero di chiamata', 'Volumi/date a cui si applica il numero di chiamata', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '050', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '050', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '050', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '052', 'a', 'Codice di classificazione dell\'area geografica', 'Codice di classificazione dell\'area geografica', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '052', 'b', 'Codice di classificazione della sottoarea geografica', 'Codice di classificazione della sottoarea geografica', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '052', 'd', 'Nome di luogo abitato', 'Nome di luogo abitato', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '052', '2', 'Fonte del codice', 'Fonte del codice', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '052', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '052', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '053', 'a', 'Elemento del numero di classificazione-numero unico o numero di inizio dell’espansione', 'Elemento del numero di classificazione-numero unico o numero di inizio dell’espansione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '053', 'b', 'Elemento del numero di classificazione-numero finale dell\'espansione', 'Elemento del numero di classificazione-numero finale dell\'espansione', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '053', 'c', 'Termine esplicativo', 'Termine esplicativo', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '053', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '053', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '053', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '055', 'a', 'Numero di classificazione', 'Numero di classificazione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '055', 'b', 'Numero d\'item', 'Numero d\'item', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '055', 'd', 'Volumi/date a cui si applica il numero di chiamata', 'Volumi/date a cui si applica il numero di chiamata', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '055', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '055', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '055', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '058', 'a', 'Numero di classificazione LC -- Numero unico o numero d\'inizio del range', 'Numero di classificazione LC -- Numero unico o numero d\'inizio del range', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '058', 'b', 'LC classification number--End number of a range', 'LC classification number--End number of a range', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '058', 'c', 'Termine esplicativo', 'Termine esplicativo', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '058', '5', 'Library to which class number applies', 'Library to which class number applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '058', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '060', 'a', 'Numero di classificazione', 'Numero di classificazione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '060', 'b', 'Numero d\'item', 'Numero d\'item', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '060', 'd', 'Volumi/date a cui si applica il numero di chiamata', 'Volumi/date a cui si applica il numero di chiamata', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '060', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '060', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '060', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '063', 'a', 'NLM classification number--Single number or beginning number of a range', 'NLM classification number--Single number or beginning number of a range', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '063', 'b', 'NLM classification number--End number of a range', 'NLM classification number--End number of a range', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '063', 'c', 'Termine esplicativo', 'Termine esplicativo', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '065', 'a', 'Elemento del numero di classificazione-numero unico o numero di inizio dell’espansione', 'Elemento del numero di classificazione-numero unico o numero di inizio dell’espansione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '065', 'b', 'Elemento del numero di classificazione-numero finale dell\'espansione', 'Elemento del numero di classificazione-numero finale dell\'espansione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '065', 'c', 'Termine esplicativo', 'Termine esplicativo', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '065', '2', 'Fonte del numero', 'Fonte del numero', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '065', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '065', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '065', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '066', 'a', 'Set di caratteri primario G0', 'Set di caratteri primario G0', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '066', 'b', 'Set di caratteri primario G1', 'Set di caratteri primario G1', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '066', 'c', 'Set di caratteri alternativo G0 o G1', 'Set di caratteri alternativo G0 o G1', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '068', 'a', 'NLM classification number--Single number or beginning number of a range', 'NLM classification number--Single number or beginning number of a range', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '068', 'b', 'NLM classification number--End number of a range', 'NLM classification number--End number of a range', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '068', 'c', 'Termine esplicativo', 'Termine esplicativo', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '068', '5', 'Library to which class number applies', 'Library to which class number applies', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '070', 'a', 'Numero di classificazione', 'Numero di classificazione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '070', 'b', 'Numero d\'item', 'Numero d\'item', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '070', 'd', 'Volumi/date a cui si applica il numero di chiamata', 'Volumi/date a cui si applica il numero di chiamata', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '070', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '070', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '072', 'a', 'Codice di categoria di soggetto', 'Codice di categoria di soggetto', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '072', 'x', 'Suddivisione del codice di categoria di soggetto', 'Suddivisione del codice di categoria di soggetto', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '072', '2', 'Fonte del codice', 'Fonte del codice', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '072', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '072', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '073', 'a', 'Impiego della suddivisione', 'Impiego della suddivisione', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '073', 'z', 'Fonte del codice', 'Fonte del codice', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '073', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '073', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '080', 'a', 'Numero di Classificazione decimale universale', 'Numero di Classificazione decimale universale', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '080', 'b', 'Numero d\'item', 'Numero d\'item', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '080', 'x', 'Suddivisione ausiliare comune', 'Suddivisione ausiliare comune', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '080', '2', 'Numero dell\'edizione', 'Numero dell\'edizione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '080', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '080', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '083', 'y', 'Numero sequenza tavola per aggiunta o ordinamento interno', 'Numero sequenza tavola per aggiunta o ordinamento interno', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '082', 'a', 'Numero di classificazione', 'Numero di classificazione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '082', 'b', 'Numero d\'item', 'Numero d\'item', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '082', 'd', 'Volumi/date a cui si applica il numero di chiamata', 'Volumi/date a cui si applica il numero di chiamata', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '082', '2', 'Numero dell\'edizione', 'Numero dell\'edizione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '082', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '082', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '082', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '083', 'a', 'Elemento del numero di classificazione-numero unico o numero di inizio dell’espansione', 'Elemento del numero di classificazione-numero unico o numero di inizio dell’espansione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '083', 'b', 'Elemento del numero di classificazione-numero finale dell\'espansione', 'Elemento del numero di classificazione-numero finale dell\'espansione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '083', 'c', 'Termine esplicativo', 'Termine esplicativo', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '083', 'z', 'Identificazione tavola', 'Identificazione tavola', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '083', '2', 'Numero dell\'edizione', 'Numero dell\'edizione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '083', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '083', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '083', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '084', 'a', 'Codice dello schema di classificazione', 'Codice dello schema di classificazione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '084', 'b', 'Titolo edizione', 'Titolo edizione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '084', 'c', 'Numero dell\'edizione', 'Numero dell\'edizione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '084', 'd', 'Fonte codice di edizione', 'Fonte codice di edizione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '084', 'e', 'Codice lingua', 'Codice lingua', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '084', 'f', 'Autorizzazione', 'Autorizzazione', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '084', 'n', 'Variazioni', 'Variazioni', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '084', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '084', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '086', 'a', 'Numero di chiamata', 'Numero di chiamata', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '086', 'd', 'Volumi/date a cui si applica il numero di chiamata', 'Volumi/date a cui si applica il numero di chiamata', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '086', 'z', 'Numero di chiamata cancellato/errato', 'Numero di chiamata cancellato/errato', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '086', '2', 'Fonte del numero', 'Fonte del numero', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '086', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '086', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '086', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '087', 'a', 'Elemento del numero di classificazione-numero unico o numero di inizio dell’espansione', 'Elemento del numero di classificazione-numero unico o numero di inizio dell’espansione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '087', 'b', 'Elemento del numero di classificazione-numero finale dell\'espansione', 'Elemento del numero di classificazione-numero finale dell\'espansione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '087', 'c', 'Informazione esplicativa', 'Informazione esplicativa', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '087', '2', 'Fonte del numero', 'Fonte del numero', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '087', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '087', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '088', 'a', 'Numero di collocazione del documento (CODOC) [OBSOLETO]', 'Numero di collocazione del documento (CODOC) [OBSOLETO]', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', 'a', 'Call number ; Classification number (OCLC) (R) ; Classification number, CALL (RLIN) (NR)', 'Call number ; Classification number (OCLC) (R) ; Classification number, CALL (RLIN) (NR)', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', 'b', 'Item number ; Local cutter number (OCLC) ; Book number/undivided call number, CALL (RLIN)', 'Item number ; Local cutter number (OCLC) ; Book number/undivided call number, CALL (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', 'd', 'Volumi/date a cui si applica il numero di chiamata', 'Volumi/date a cui si applica il numero di chiamata', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', 'e', 'Feature heading (OCLC)', 'Feature heading (OCLC)', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', 'f', 'Filing suffix (OCLC) ; Footnote, FNT (RLIN)', 'Filing suffix (OCLC) ; Footnote, FNT (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', 'h', 'Output transaction history, HST (RLIN)', 'Output transaction history, HST (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', 'i', 'Output transaction instruction, INS (RLIN)', 'Output transaction instruction, INS (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', 'l', 'Extra card control statement, EXT (RLIN)', 'Extra card control statement, EXT (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', 'n', 'Additional local notes, ANT (RLIN)', 'Additional local notes, ANT (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', 'p', 'Pathfinder code, PTH (RLIN)', 'Pathfinder code, PTH (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', 't', 'Field suppresion, FSP (RLIN)', 'Field suppresion, FSP (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', 'v', 'Volumes, VOL (RLIN)', 'Volumes, VOL (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', 'y', 'Date, VOL (RLIN)', 'Date, VOL (RLIN)', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', 'z', 'Numero di chiamata cancellato/errato', 'Numero di chiamata cancellato/errato', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', '2', 'Fonte del numero', 'Fonte del numero', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '090', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '091', 'a', 'Elemento del numero di classificazione-numero unico o numero di inizio dell’espansione', 'Elemento del numero di classificazione-numero unico o numero di inizio dell’espansione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '091', 'b', 'Elemento del numero di classificazione-numero finale dell\'espansione', 'Elemento del numero di classificazione-numero finale dell\'espansione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '091', 'c', 'Termine esplicativo', 'Termine esplicativo', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '091', 'z', 'Identificazione tavola -- numero tavola', 'Identificazione tavola -- numero tavola', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '091', '2', 'Numero dell\'edizione', 'Numero dell\'edizione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '091', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '091', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '091', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '092', 'a', 'Numero di classificazione', 'Numero di classificazione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '092', 'b', 'Numero d\'item', 'Numero d\'item', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '092', 'd', 'Volumi/date a cui si applica il numero di chiamata', 'Volumi/date a cui si applica il numero di chiamata', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '092', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '092', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '092', '2', 'Numero dell\'edizione', 'Numero dell\'edizione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '092', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '092', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '092', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '093', 'a', 'Elemento del numero di classificazione-numero unico o numero di inizio dell’espansione', 'Elemento del numero di classificazione-numero unico o numero di inizio dell’espansione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '093', 'b', 'Elemento del numero di classificazione-numero finale dell\'espansione', 'Elemento del numero di classificazione-numero finale dell\'espansione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '093', 'c', 'Termine esplicativo', 'Termine esplicativo', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '093', 'z', 'Identificazione tavola -- numero tavola', 'Identificazione tavola -- numero tavola', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '093', '2', 'Numero dell\'edizione', 'Numero dell\'edizione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '093', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '093', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '093', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '096', 'a', 'Numero di classificazione', 'Numero di classificazione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '096', 'b', 'Numero d\'item', 'Numero d\'item', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '096', 'd', 'Volumi/date a cui si applica il numero di chiamata', 'Volumi/date a cui si applica il numero di chiamata', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '096', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '096', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '096', '2', 'Numero dell\'edizione', 'Numero dell\'edizione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '096', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '096', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '096', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '097', 'a', 'Elemento del numero di classificazione-numero unico o numero di inizio dell’espansione', 'Elemento del numero di classificazione-numero unico o numero di inizio dell’espansione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '097', 'b', 'Elemento del numero di classificazione-numero finale dell\'espansione', 'Elemento del numero di classificazione-numero finale dell\'espansione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '097', 'c', 'Termine esplicativo', 'Termine esplicativo', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '097', 'z', 'Identificazione tavola -- numero tavola', 'Identificazione tavola -- numero tavola', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '097', '2', 'Numero dell\'edizione', 'Numero dell\'edizione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '097', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '097', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '097', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '098', 'a', 'Call number based on other classification scheme', 'Call number based on other classification scheme', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '098', 'b', 'Numero d\'item', 'Numero d\'item', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '098', 'd', 'Volumi/date a cui si applica il numero di chiamata', 'Volumi/date a cui si applica il numero di chiamata', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '098', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '098', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '098', '2', 'Numero dell\'edizione', 'Numero dell\'edizione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '098', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '098', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '098', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '099', 'a', 'Numero di classificazione', 'Numero di classificazione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '099', 'b', 'Numero d\'item', 'Numero d\'item', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '099', 'd', 'Volumi/date a cui si applica il numero di chiamata', 'Volumi/date a cui si applica il numero di chiamata', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '099', 'e', 'Feature heading', 'Feature heading', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '099', 'f', 'Filing suffix', 'Filing suffix', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '099', '2', 'Numero dell\'edizione', 'Numero dell\'edizione', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '099', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '099', '6', 'Collegamento', 'Collegamento', 0, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '099', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 0, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'a', 'Nome di persona', 'Nome di persona', 0, 0, 1, NULL, NULL, '''100b'',''100c'',''100q'',''100d'',''100t'',''100o'',''100m'',''100r'',''100s'',''100k'',''100n'',''100p'',''100g'',''100l'',''100f'',''100h'',''100x'',''100z'',''100y'',''100v''', 0, 0, '', '', ''),
		('', '100', 'b', 'Numerazione', 'Numerazione', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'c', 'Titoli e altre parole associate al nome', 'Titoli e altre parole associate al nome', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'd', 'Date associate al nome', 'Date associate al nome', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'e', 'Termine di ruolo', 'Termine di ruolo', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'f', 'Data dell\'opera', 'Data dell\'opera', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'g', 'Informazione miscellanea', 'Informazione miscellanea', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'h', 'Supporto', 'Supporto', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'j', 'Qualificazione d\'attribuzione', 'Qualificazione d\'attribuzione', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'k', 'Sottointestazione formale', 'Sottointestazione formale', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'l', 'Lingua dell\'opera', 'Lingua dell\'opera', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'm', 'Mezzo di esecuzione per la musica', 'Mezzo di esecuzione per la musica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'n', 'Numero di parte/sezione dell\'opera', 'Numero di parte/sezione dell\'opera', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'o', 'Formulazione d\'arrangiamento per la musica', 'Formulazione d\'arrangiamento per la musica', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'p', 'Nome di parte/sezione dell\'opera', 'Nome di parte/sezione dell\'opera', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'q', 'Forma completa del nome', 'Forma completa del nome', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'r', 'Tonalità (musica)', 'Tonalità (musica)', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 's', 'Versione', 'Versione', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 't', 'Titolo dell\'opera', 'Titolo dell\'opera', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', '6', 'Collegamento', 'Collegamento', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '100', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'a', 'Nome dell\'ente o nome della giurisdizione come accesso', 'Nome dell\'ente o nome della giurisdizione come accesso', 0, 0, 1, NULL, NULL, '''110b'',''110c'',''110d'',''110t'',''110o'',''110m'',''110r'',''110s'',''110k'',''110n'',''110p'',''110g'',''110l'',''110f'',''110h'',''110x'',''110z'',''110y'',''110v''', 0, 0, '', '', ''),
		('', '110', 'b', 'Unità subordinata', 'Unità subordinata', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'c', 'Luogo del convegno', 'Luogo del convegno', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'd', 'Data del convegno o di firma del trattato', 'Data del convegno o di firma del trattato', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'e', 'Termine di ruolo', 'Termine di ruolo', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'f', 'Data dell\'opera', 'Data dell\'opera', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'g', 'Informazione miscellanea', 'Informazione miscellanea', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'h', 'Supporto', 'Supporto', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'k', 'Sottointestazione formale', 'Sottointestazione formale', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'l', 'Lingua dell\'opera', 'Lingua dell\'opera', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'm', 'Mezzo di esecuzione per la musica', 'Mezzo di esecuzione per la musica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'n', 'Numero di parte/sezione/convegno', 'Numero di parte/sezione/convegno', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'o', 'Formulazione d\'arrangiamento per la musica', 'Formulazione d\'arrangiamento per la musica', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'p', 'Nome di parte/sezione dell\'opera', 'Nome di parte/sezione dell\'opera', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'r', 'Tonalità (musica)', 'Tonalità (musica)', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 's', 'Versione', 'Versione', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 't', 'Titolo dell\'opera', 'Titolo dell\'opera', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', '6', 'Collegamento', 'Collegamento', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '110', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 'a', 'Nome del convegno o nome della giurisdizione come accesso', 'Nome del convegno o nome della giurisdizione come accesso', 0, 0, 1, NULL, NULL, '''111e'',''111c'',''111d'',''111t'',''111s'',''111k'',''111n'',''111p'',''111g'',''111l'',''111f'',''111h'',''111x'',''111z'',''111y'',''111v''', 0, 0, '', '', ''),
		('', '111', 'b', 'Number [OBSOLETE]', 'Number [OBSOLETE]', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 'c', 'Luogo del convegno', 'Luogo del convegno', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 'd', 'Data del convegno', 'Data del convegno', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 'e', 'Unità subordinata', 'Unità subordinata', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 'f', 'Data dell\'opera', 'Data dell\'opera', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 'g', 'Informazione miscellanea', 'Informazione miscellanea', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 'h', 'Supporto', 'Supporto', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 'j', 'Termine di ruolo', 'Termine di ruolo', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 'k', 'Sottointestazione formale', 'Sottointestazione formale', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 'l', 'Lingua dell\'opera', 'Lingua dell\'opera', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 'n', 'Numero di parte/sezione/convegno', 'Numero di parte/sezione/convegno', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 'p', 'Nome di parte/sezione dell\'opera', 'Nome di parte/sezione dell\'opera', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 'q', 'Nome del convegno dopo accesso di nome della giurisdizione', 'Nome del convegno dopo accesso di nome della giurisdizione', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 's', 'Versione', 'Versione', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 't', 'Titolo dell\'opera', 'Titolo dell\'opera', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', '6', 'Collegamento', 'Collegamento', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '111', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', 'a', 'Titolo uniforme', 'Titolo uniforme', 0, 0, 1, NULL, NULL, '''130o'',''130m'',''130r'',''130s'',''130d'',''130k'',''130n'',''130p'',''130g'',''130l'',''130f'',''130h'',''130t'',''130x'',''130z'',''130y'',''130v''', 0, 0, '', '', ''),
		('', '130', 'd', 'Data di firma del trattato', 'Data di firma del trattato', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', 'f', 'Data dell\'opera', 'Data dell\'opera', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', 'g', 'Informazione miscellanea', 'Informazione miscellanea', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', 'h', 'Supporto', 'Supporto', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', 'k', 'Sottointestazione formale', 'Sottointestazione formale', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', 'l', 'Lingua dell\'opera', 'Lingua dell\'opera', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', 'm', 'Mezzo di esecuzione per la musica', 'Mezzo di esecuzione per la musica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', 'n', 'Numero di parte/sezione dell\'opera', 'Numero di parte/sezione dell\'opera', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', 'o', 'Formulazione d\'arrangiamento per la musica', 'Formulazione d\'arrangiamento per la musica', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', 'p', 'Nome di parte/sezione dell\'opera', 'Nome di parte/sezione dell\'opera', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', 'r', 'Tonalità (musica)', 'Tonalità (musica)', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', 's', 'Versione', 'Versione', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', 't', 'Titolo dell\'opera', 'Titolo dell\'opera', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', '6', 'Collegamento', 'Collegamento', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '130', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '140', 'a', 'Titolo uniforme', 'Titolo uniforme', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '140', 'd', 'Data del trattato [OBSOLETO]', 'Data del trattato [OBSOLETO]', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '140', 'f', 'Data dell\'opera', 'Data dell\'opera', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '140', 'g', 'Informazione miscellanea', 'Informazione miscellanea', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '140', 'h', 'Designazione generale del materiale [OBSOLETO] ', 'Designazione generale del materiale [OBSOLETO] ', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '140', 'k', 'Sottointestazione formale', 'Sottointestazione formale', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '140', 'l', 'Lingua [OBSOLETO] ', 'Lingua [OBSOLETO] ', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '140', 'm', 'Mezzo di esecuzione per la musica', 'Mezzo di esecuzione per la musica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '140', 'n', 'Numero di parte o sezione/ordine, tematica, o numero dell’opus  [OBSOLETO] ', 'Numero di parte o sezione/ordine, tematica, o numero dell’opus  [OBSOLETO] ', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '140', 'o', 'Arrangiato o arr. per la musica  [OBSOLETO] ', 'Arrangiato o arr. per la musica  [OBSOLETO] ', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '140', 'p', 'Parte o sezione [OBSOLETO] ', 'Parte o sezione [OBSOLETO] ', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '140', 'r', 'Tonalità (musica)', 'Tonalità (musica)', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '140', 'x', 'Suddivisione generale di soggetto  [OBSOLETO] ', 'Suddivisione generale di soggetto  [OBSOLETO] ', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '140', 'y', 'Suddivisione di tempo di soggetto [OBSOLETO]', 'Suddivisione di tempo di soggetto [OBSOLETO]', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '140', 'z', 'Suddivisione di luogo di soggetto [OBSOLETO] ', 'Suddivisione di luogo di soggetto [OBSOLETO] ', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '143', 'a', 'Titolo collettivo  [OBSOLETO]', 'Titolo collettivo  [OBSOLETO]', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '143', 'd', 'Data del trattato [OBSOLETO]', 'Data del trattato [OBSOLETO]', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '143', 'f', 'Data dell\'opera', 'Data dell\'opera', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '143', 'g', 'Informazione miscellanea', 'Informazione miscellanea', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '143', 'h', 'Designazione generale del materiale [OBSOLETO] ', 'Designazione generale del materiale [OBSOLETO] ', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '143', 'k', 'Sottointestazione formale', 'Sottointestazione formale', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '143', 'l', 'Lingua [OBSOLETO] ', 'Lingua [OBSOLETO] ', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '143', 'm', 'Mezzo di esecuzione per la musica', 'Mezzo di esecuzione per la musica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '143', 'n', 'Numero di parte o sezione/ordine, tematica, o numero dell’opus  [OBSOLETO] ', 'Numero di parte o sezione/ordine, tematica, o numero dell’opus  [OBSOLETO] ', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '143', 'o', 'Arrangiato o arr. per la musica  [OBSOLETO] ', 'Arrangiato o arr. per la musica  [OBSOLETO] ', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '143', 'p', 'Parte o sezione [OBSOLETO] ', 'Parte o sezione [OBSOLETO] ', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '143', 'r', 'Tonalità (musica)', 'Tonalità (musica)', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '143', 's', 'Versione', 'Versione', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '148', 'a', 'Termine cronologico', 'Termine cronologico', 0, 0, 1, NULL, NULL, '''148y'',''148x'',''148z'',''148v''', 0, 0, '', '', ''),
		('', '148', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '148', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '148', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '148', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '148', '6', 'Collegamento', 'Collegamento', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '148', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '150', 'a', 'Termine topico o nome geografico come accesso', 'Termine topico o nome geografico come accesso', 0, 0, 1, NULL, NULL, '''150x'',''150z'',''150y'',''150v''', 0, 0, '', '', ''),
		('', '150', 'b', 'Termine topico che segue il nome geografico come accesso', 'Termine topico che segue il nome geografico come accesso', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '150', 'g', 'Informazione miscellanea', 'Informazione miscellanea', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '150', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '150', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '150', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '150', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '150', '6', 'Collegamento', 'Collegamento', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '150', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '151', 'a', 'Nome geografico', 'Nome geografico', 0, 0, 1, NULL, NULL, '''151z'',''151x'',''151y'',''151v''', 0, 0, '', '', ''),
		('', '151', 'b', 'Name following place as an entry element {OBSOLETE]', 'Name following place as an entry element {OBSOLETE]', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '151', 'g', 'Informazione miscellanea', 'Informazione miscellanea', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '151', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '151', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '151', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '151', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '151', '6', 'Collegamento', 'Collegamento', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '151', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '153', 'a', 'Classification number--Single number or beginning number of span', 'Classification number--Single number or beginning number of span', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '153', 'c', 'Numero di classificazione – fine intervallo', 'Numero di classificazione – fine intervallo', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '153', 'h', 'Caption hierarchy', 'Caption hierarchy', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '153', 'j', 'Caption', 'Caption', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '153', 'k', 'Summary number span caption hierarchy', 'Summary number span caption hierarchy', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '153', 'z', 'Identificazione tavola', 'Identificazione tavola', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '153', '6', 'Collegamento', 'Collegamento', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '153', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '154', 'a', 'General explanatory index term', 'General explanatory index term', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '154', 'b', 'General explanatory index term--Succeeding level', 'General explanatory index term--Succeeding level', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '154', 'f', 'Schedule identification', 'Schedule identification', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '154', '6', 'Collegamento', 'Collegamento', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '154', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '155', 'a', 'Termine di genere/forma', 'Termine di genere/forma', 0, 0, 1, NULL, NULL, '''155v'',''155x'',''155z'',''155y''', 0, 0, '', '', ''),
		('', '155', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '155', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '155', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '155', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '155', '6', 'Collegamento', 'Collegamento', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '155', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '162', 'a', 'Termine del mezzo di esecuzione', 'Termine del mezzo di esecuzione', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '162', '6', 'Collegamento', 'Collegamento', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '162', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '180', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '180', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '180', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '180', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '180', '6', 'Collegamento', 'Collegamento', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '180', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '181', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '181', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '181', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '181', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '181', '6', 'Collegamento', 'Collegamento', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '181', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '182', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '182', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '182', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '182', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '182', '6', 'Collegamento', 'Collegamento', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '182', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '185', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '185', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '185', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '185', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '185', '6', 'Collegamento', 'Collegamento', 0, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '185', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 1, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '253', 'a', 'Classification number referred to--Single number or beginning number of span', 'Classification number referred to--Single number or beginning number of span', 1, 0, 2, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '253', 'c', 'Classification number referred to--Ending number of span', 'Classification number referred to--Ending number of span', 1, 0, 2, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '253', 'i', 'Spiegazione', 'Spiegazione', 1, 0, 2, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '253', 'y', 'Table identification--Schedule [OBSOLETE]', 'Table identification--Schedule [OBSOLETE]', 0, 0, 2, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '253', 'z', 'Identificazione tavola', 'Identificazione tavola', 1, 0, 2, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '253', '6', 'Collegamento', 'Collegamento', 0, 0, 2, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '253', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 2, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '260', 'a', 'Luogo di pubblicazione, distribuzione, ecc.', 'Luogo di pubblicazione, distribuzione, ecc.', 1, 0, 2, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '260', 'i', 'Spiegazione', 'Spiegazione', 1, 0, 2, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '260', '0', 'Numero del record di authority o numero standard', 'Numero del record di authority o numero standard', 1, 0, 2, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '260', '6', 'Collegamento', 'Collegamento', 0, 0, 2, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '260', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 2, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '336', 'a', 'Termine del tipo di contenuto', 'Termine del tipo di contenuto', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '336', 'b', 'Codice del tipo di contenuto', 'Codice del tipo di contenuto', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '336', '0', 'Numero del record di authority o numero standard', 'Numero del record di authority o numero standard', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '336', '2', 'Fonte', 'Fonte', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '336', '3', 'Materiale specificato', 'Materiale specificato', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '336', '6', 'Collegamento', 'Collegamento', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '336', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '348', 'a', 'Presentazione musicale - termine', 'Presentazione musicale - termine', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '348', 'b', 'Presentazione musicale -  codice', 'Presentazione musicale -  codice', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '348', '0', 'Numero del record di authority o numero standard', 'Numero del record di authority o numero standard', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '348', '2', 'Fonte', 'Fonte', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '348', '3', 'Materiale specificato', 'Materiale specificato', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '348', '6', 'Collegamento', 'Collegamento', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '348', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '353', 'a', 'Classification number referred to--Single number or beginning number of span', 'Classification number referred to--Single number or beginning number of span', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '353', 'c', 'Classification number referred to--Ending number of span', 'Classification number referred to--Ending number of span', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '353', 'i', 'Spiegazione', 'Spiegazione', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '353', 'y', 'Table identification--Schedule [OBSOLETE]', 'Table identification--Schedule [OBSOLETE]', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '353', 'z', 'Identificazione tavola', 'Identificazione tavola', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '353', '6', 'Collegamento', 'Collegamento', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '353', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '360', 'a', 'Luogo di pubblicazione, distribuzione, ecc.', 'Luogo di pubblicazione, distribuzione, ecc.', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '360', 'i', 'Spiegazione', 'Spiegazione', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '360', '0', 'Numero del record di authority o numero standard', 'Numero del record di authority o numero standard', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '360', '6', 'Collegamento', 'Collegamento', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '360', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '368', 'a', 'Tipo dell\'ente', 'Tipo dell\'ente', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '368', 'b', 'Tipo di giurisdizione', 'Tipo di giurisdizione', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '368', 'c', 'Altra designazione', 'Altra designazione', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '368', 'd', 'Titolo della persona', 'Titolo della persona', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '368', 's', 'Inizio del periodo', 'Inizio del periodo', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '368', 't', 'Fine del periodo', 'Fine del periodo', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '368', 'u', 'URI (Uniform Resource Identifier)', 'URI (Uniform Resource Identifier)', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '368', 'v', 'Fonte dell\'informazione', 'Fonte dell\'informazione', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '368', '0', 'Numero del record di authority o numero standard', 'Numero del record di authority o numero standard', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '368', '2', 'Fonte', 'Fonte', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '368', '6', 'Collegamento', 'Collegamento', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '368', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '370', 'a', 'Luogo di nascita', 'Luogo di nascita', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '370', 'b', 'Luogo di morte', 'Luogo di morte', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '370', 'c', 'Paese associato', 'Paese associato', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '370', 'e', 'Luogo di residenza/sede', 'Luogo di residenza/sede', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '370', 'f', 'Altro luogo associato', 'Altro luogo associato', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '370', 'g', 'Luogo di origine di un’opera', 'Luogo di origine di un’opera', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '370', 's', 'Inizio del periodo', 'Inizio del periodo', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '370', 't', 'Fine del periodo', 'Fine del periodo', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '370', 'u', 'URI (Uniform Resource Identifier)', 'URI (Uniform Resource Identifier)', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '370', 'v', 'Fonte dell\'informazione', 'Fonte dell\'informazione', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '370', '0', 'Numero di controllo della registrazione', 'Numero di controllo della registrazione', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '370', '2', 'Fonte', 'Fonte', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '370', '6', 'Collegamento', 'Collegamento', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '370', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '371', 'a', 'Indirizzo', 'Indirizzo', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '371', 'b', 'Città', 'Città', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '371', 'c', 'Ente territoriale intermedio', 'Ente territoriale intermedio', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '371', 'd', 'Paese', 'Paese', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '371', 'e', 'Codice postale', 'Codice postale', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '371', 'm', 'Indirizzo e-mail', 'Indirizzo e-mail', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '371', 's', 'Inizio del periodo', 'Inizio del periodo', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '371', 't', 'Fine del periodo', 'Fine del periodo', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '371', 'u', 'URI (Uniform Resource Identifier)', 'URI (Uniform Resource Identifier)', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '371', 'v', 'Fonte dell\'informazione', 'Fonte dell\'informazione', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '371', 'z', 'Nota pubblica', 'Nota pubblica', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '371', '4', 'Codice di ruolo', 'Codice di ruolo', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '371', '6', 'Collegamento', 'Collegamento', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '371', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '372', 'a', 'Campo di attività', 'Campo di attività', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '372', 's', 'Inizio del periodo', 'Inizio del periodo', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '372', 't', 'Fine del periodo', 'Fine del periodo', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '372', 'u', 'URI (Uniform Resource Identifier)', 'URI (Uniform Resource Identifier)', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '372', 'v', 'Fonte dell\'informazione', 'Fonte dell\'informazione', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '372', '0', 'Numero di controllo della registrazione', 'Numero di controllo della registrazione', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '372', '2', 'Fonte', 'Fonte', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '372', '6', 'Collegamento', 'Collegamento', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '372', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '373', 'a', 'Affiliazione', 'Affiliazione', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '373', 's', 'Inizio del periodo', 'Inizio del periodo', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '373', 't', 'Fine del periodo', 'Fine del periodo', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '373', 'u', 'URI (Uniform Resource Identifier)', 'URI (Uniform Resource Identifier)', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '373', 'v', 'Fonte dell\'informazione', 'Fonte dell\'informazione', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '373', '0', 'Numero di controllo della registrazione', 'Numero di controllo della registrazione', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '373', '2', 'Fonte', 'Fonte', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '373', '6', 'Collegamento', 'Collegamento', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '373', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '374', 'a', 'Occupazione', 'Occupazione', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '374', 's', 'Inizio del periodo', 'Inizio del periodo', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '374', 't', 'Fine del periodo', 'Fine del periodo', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '374', 'u', 'URI (Uniform Resource Identifier)', 'URI (Uniform Resource Identifier)', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '374', 'v', 'Fonte dell\'informazione', 'Fonte dell\'informazione', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '374', '0', 'Numero di controllo della registrazione', 'Numero di controllo della registrazione', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '374', '2', 'Fonte', 'Fonte', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '374', '6', 'Collegamento', 'Collegamento', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '374', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '375', 'a', 'Sesso', 'Sesso', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '375', 's', 'Inizio del periodo', 'Inizio del periodo', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '375', 't', 'Fine del periodo', 'Fine del periodo', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '375', 'u', 'URI (Uniform Resource Identifier)', 'URI (Uniform Resource Identifier)', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '375', 'v', 'Fonte dell\'informazione', 'Fonte dell\'informazione', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '375', '2', 'Fonte', 'Fonte', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '375', '6', 'Collegamento', 'Collegamento', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '375', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '376', 'a', 'Tipo di famiglia', 'Tipo di famiglia', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '376', 'b', 'Nome del membro prominente', 'Nome del membro prominente', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '376', 'c', 'Titolo ereditario', 'Titolo ereditario', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '376', 's', 'Inizio del periodo', 'Inizio del periodo', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '376', 't', 'Fine del periodo', 'Fine del periodo', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '376', 'u', 'URI (Uniform Resource Identifier)', 'URI (Uniform Resource Identifier)', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '376', 'v', 'Fonte dell\'informazione', 'Fonte dell\'informazione', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '376', '0', 'Numero di controllo della registrazione', 'Numero di controllo della registrazione', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '376', '2', 'Fonte', 'Fonte', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '376', '6', 'Collegamento', 'Collegamento', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '376', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '377', 'a', 'Codice lingua', 'Codice lingua', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '377', 'l', 'Lingua per esteso', 'Lingua per esteso', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '377', '2', 'Fonte del codice', 'Fonte del codice', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '377', '6', 'Collegamento', 'Collegamento', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '377', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '378', 'q', 'Nome di persona - forma espansa', 'Nome di persona - forma espansa', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '378', 'u', 'URI (Uniform Resource Identifier)', 'URI (Uniform Resource Identifier)', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '378', 'v', 'Fonte dell\'informazione', 'Fonte dell\'informazione', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '378', '6', 'Collegamento', 'Collegamento', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '378', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '380', 'a', 'Forma dell\'emissione', 'Forma dell\'emissione', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '380', '0', 'Numero di controllo della registrazione', 'Numero di controllo della registrazione', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '380', '2', 'Fonte', 'Fonte', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '380', '6', 'Collegamento', 'Collegamento', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '380', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '381', 'a', 'Altre caratteristiche distintive', 'Altre caratteristiche distintive', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '381', 'u', 'URI (Uniform Resource Identifier)', 'URI (Uniform Resource Identifier)', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '381', 'v', 'Fonte dell\'informazione', 'Fonte dell\'informazione', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '381', '0', 'Numero di controllo della registrazione', 'Numero di controllo della registrazione', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '381', '2', 'Fonte', 'Fonte', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '381', '6', 'Collegamento', 'Collegamento', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '381', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '382', 'a', 'Modalità dell\'esecuzione', 'Modalità dell\'esecuzione', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '382', 'b', 'Solista', 'Solista', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '382', 'd', 'Secondo strumento', 'Secondo strumento', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '382', 'e', 'Numero di complessi', 'Numero di complessi', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '382', 'n', 'Numero di esecutori con stesso strumento', 'Numero di esecutori con stesso strumento', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '382', 'p', 'Strumento alternativo', 'Strumento alternativo', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '382', 's', 'Numero totale di esecutori', 'Numero totale di esecutori', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '382', 'v', 'Nota', 'Nota', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '382', '0', 'Numero del record di authority o numero standard', 'Numero del record di authority o numero standard', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '382', '2', 'Fonte', 'Fonte', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '382', '6', 'Collegamento', 'Collegamento', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '382', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '383', 'a', 'Numero seriale', 'Numero seriale', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '383', 'b', 'Numero nell\'opus', 'Numero nell\'opus', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '383', 'c', 'Numero indice tematico', 'Numero indice tematico', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '383', 'd', 'Codice indice tematico', 'Codice indice tematico', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '383', 'e', 'Editore associato a numero nell\'opus', 'Editore associato a numero nell\'opus', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '383', '2', 'Fonte', 'Fonte', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '383', '6', 'Collegamento', 'Collegamento', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '383', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '384', 'a', 'Chiave musicale', 'Chiave musicale', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '384', '6', 'Collegamento', 'Collegamento', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '384', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '385', 'a', 'Pubblico', 'Pubblico', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '385', 'b', 'Pubblico (codice)', 'Pubblico (codice)', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '385', 'm', 'Gruppo demografico', 'Gruppo demografico', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '385', 'n', 'Gruppo demografico (codice)', 'Gruppo demografico (codice)', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '385', '0', 'Numero del record di authority o numero standard', 'Numero del record di authority o numero standard', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '385', '2', 'Fonte', 'Fonte', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '385', '3', 'Materiale specificato', 'Materiale specificato', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '385', '6', 'Collegamento', 'Collegamento', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '385', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '386', 'a', 'Creatore /contributore', 'Creatore /contributore', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '386', 'b', 'Creatore/contributore (codice)', 'Creatore/contributore (codice)', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '386', 'm', 'Gruppo demografico', 'Gruppo demografico', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '386', 'n', 'Gruppo demografico (codice)', 'Gruppo demografico (codice)', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '386', '0', 'Numero del record di authority o numero standard', 'Numero del record di authority o numero standard', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '386', '2', 'Fonte', 'Fonte', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '386', '3', 'Materiale specificato', 'Materiale specificato', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '386', '6', 'Collegamento', 'Collegamento', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '386', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '388', '0', 'Numero del record di authority o numero standard', 'Numero del record di authority o numero standard', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '388', '2', 'Fonte', 'Fonte', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '388', '3', 'Materiale specificato', 'Materiale specificato', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '388', '6', 'Collegamento', 'Collegamento', 0, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '388', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '388', 'a', 'Termine per il periodo della creazione del contenuto', 'Termine per il periodo della creazione del contenuto', 1, 0, 3, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'a', 'Nome di persona', 'Nome di persona', 0, 0, 4, NULL, NULL, '''400b'',''400c'',''400q'',''400d'',''400t'',''400o'',''400m'',''400r'',''400s'',''400k'',''400n'',''400p'',''400g'',''400l'',''400f'',''400h'',''400x'',''400z'',''400y'',''400v''', 0, 0, '', '', 'PERSO_NAME'),
		('', '400', 'b', 'Numerazione', 'Numerazione', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'c', 'Titoli e altre parole associate al nome', 'Titoli e altre parole associate al nome', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'd', 'Date associate al nome', 'Date associate al nome', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'e', 'Termine di ruolo', 'Termine di ruolo', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'f', 'Data dell\'opera', 'Data dell\'opera', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'g', 'Informazione miscellanea', 'Informazione miscellanea', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'h', 'Supporto', 'Supporto', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'j', 'Qualificazione d\'attribuzione', 'Qualificazione d\'attribuzione', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'k', 'Sottointestazione formale', 'Sottointestazione formale', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'l', 'Lingua dell\'opera', 'Lingua dell\'opera', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'm', 'Mezzo di esecuzione per la musica', 'Mezzo di esecuzione per la musica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'n', 'Numero di parte/sezione dell\'opera', 'Numero di parte/sezione dell\'opera', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'o', 'Formulazione d\'arrangiamento per la musica', 'Formulazione d\'arrangiamento per la musica', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'p', 'Nome di parte/sezione dell\'opera', 'Nome di parte/sezione dell\'opera', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'q', 'Forma completa del nome', 'Forma completa del nome', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'r', 'Tonalità (musica)', 'Tonalità (musica)', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 's', 'Versione', 'Versione', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 't', 'Titolo dell\'opera', 'Titolo dell\'opera', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', '4', 'Codice relazione', 'Codice relazione', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', '6', 'Collegamento', 'Collegamento', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '400', '9', '9 (RLIN)', '9 (RLIN)', 0, 0, 4, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '410', 'a', 'Nome dell\'ente o nome della giurisdizione come accesso', 'Nome dell\'ente o nome della giurisdizione come accesso', 0, 0, 4, NULL, NULL, '''410b'',''410c'',''410d'',''410t'',''410o'',''410m'',''410r'',''410s'',''410k'',''410n'',''410p'',''410g'',''410l'',''410f'',''410h'',''410x'',''410z'',''410y'',''410v''', 0, 0, '', '', 'CORPO_NAME'),
		('', '410', 'b', 'Unità subordinata', 'Unità subordinata', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'c', 'Luogo del convegno', 'Luogo del convegno', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'd', 'Data del convegno o di firma del trattato', 'Data del convegno o di firma del trattato', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'e', 'Termine di ruolo', 'Termine di ruolo', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'f', 'Data dell\'opera', 'Data dell\'opera', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'g', 'Informazione miscellanea', 'Informazione miscellanea', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'h', 'Supporto', 'Supporto', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'k', 'Sottointestazione formale', 'Sottointestazione formale', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'l', 'Lingua dell\'opera', 'Lingua dell\'opera', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'm', 'Mezzo di esecuzione per la musica', 'Mezzo di esecuzione per la musica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'n', 'Numero di parte/sezione/convegno', 'Numero di parte/sezione/convegno', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'o', 'Formulazione d\'arrangiamento per la musica', 'Formulazione d\'arrangiamento per la musica', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'p', 'Nome di parte/sezione dell\'opera', 'Nome di parte/sezione dell\'opera', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'r', 'Tonalità (musica)', 'Tonalità (musica)', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 's', 'Versione', 'Versione', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 't', 'Titolo dell\'opera', 'Titolo dell\'opera', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', '4', 'Codice relazione', 'Codice relazione', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', '6', 'Collegamento', 'Collegamento', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '410', '9', '9 (RLIN)', '9 (RLIN)', 0, 0, 4, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '411', 'a', 'Nome del convegno o nome della giurisdizione come accesso', 'Nome del convegno o nome della giurisdizione come accesso', 0, 0, 4, NULL, NULL, '''411e'',''411c'',''411d'',''411t'',''411s'',''411k'',''411n'',''411p'',''411g'',''411l'',''411f'',''411h'',''411x'',''411z'',''411y'',''411v''', 0, 0, '', '', 'MEETI_NAME'),
		('', '411', 'b', 'Number {OBSOLETE]', 'Number {OBSOLETE]', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'c', 'Luogo del convegno', 'Luogo del convegno', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'd', 'Data del convegno', 'Data del convegno', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'e', 'Unità subordinata', 'Unità subordinata', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'f', 'Data dell\'opera', 'Data dell\'opera', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'g', 'Informazione miscellanea', 'Informazione miscellanea', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'h', 'Supporto', 'Supporto', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'j', 'Termine di ruolo', 'Termine di ruolo', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'k', 'Sottointestazione formale', 'Sottointestazione formale', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'l', 'Lingua dell\'opera', 'Lingua dell\'opera', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'n', 'Numero di parte/sezione/convegno', 'Numero di parte/sezione/convegno', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'p', 'Nome di parte/sezione dell\'opera', 'Nome di parte/sezione dell\'opera', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'q', 'Nome del convegno dopo accesso di nome della giurisdizione', 'Nome del convegno dopo accesso di nome della giurisdizione', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 's', 'Versione', 'Versione', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 't', 'Titolo dell\'opera', 'Titolo dell\'opera', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', '4', 'Codice relazione', 'Codice relazione', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', '6', 'Collegamento', 'Collegamento', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '411', '9', '9 (RLIN)', '9 (RLIN)', 0, 0, 4, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '430', 'a', 'Titolo uniforme', 'Titolo uniforme', 0, 0, 4, NULL, NULL, '''430o'',''430m'',''430r'',''430s'',''430d'',''430k'',''430n'',''430p'',''430g'',''430l'',''430f'',''430h'',''430t'',''430x'',''430z'',''430y'',''430v''', 0, 0, '', '', 'UNIF_TITLE'),
		('', '430', 'd', 'Data di firma del trattato', 'Data di firma del trattato', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 'f', 'Data dell\'opera', 'Data dell\'opera', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 'g', 'Informazione miscellanea', 'Informazione miscellanea', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 'h', 'Supporto', 'Supporto', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 'k', 'Sottointestazione formale', 'Sottointestazione formale', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 'l', 'Lingua dell\'opera', 'Lingua dell\'opera', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 'm', 'Mezzo di esecuzione per la musica', 'Mezzo di esecuzione per la musica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 'n', 'Numero di parte/sezione dell\'opera', 'Numero di parte/sezione dell\'opera', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 'o', 'Formulazione d\'arrangiamento per la musica', 'Formulazione d\'arrangiamento per la musica', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 'p', 'Nome di parte/sezione dell\'opera', 'Nome di parte/sezione dell\'opera', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 'r', 'Tonalità (musica)', 'Tonalità (musica)', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 's', 'Versione', 'Versione', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 't', 'Titolo dell\'opera', 'Titolo dell\'opera', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', '4', 'Codice relazione', 'Codice relazione', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', '6', 'Collegamento', 'Collegamento', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '430', '9', '9 (RLIN)', '9 (RLIN)', 0, 0, 4, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '448', 'a', 'Termine cronologico', 'Termine cronologico', 0, 0, 4, NULL, NULL, '''448y'',''448x'',''448z'',''448v''', 0, 0, '', '', 'CHRON_TERM'),
		('', '448', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '448', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '448', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '448', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '448', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '448', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '448', '4', 'Codice relazione', 'Codice relazione', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '448', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '448', '6', 'Collegamento', 'Collegamento', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '448', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '448', '9', '9 (RLIN)', '9 (RLIN)', 0, 0, 4, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '450', 'a', 'Termine topico o nome geografico come accesso', 'Termine topico o nome geografico come accesso', 0, 0, 4, NULL, NULL, '''450x'',''450z'',''450y'',''450v''', 0, 0, '', '', 'TOPIC_TERM'),
		('', '450', 'b', 'Termine topico che segue il nome geografico come accesso', 'Termine topico che segue il nome geografico come accesso', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '450', 'g', 'Informazione miscellanea', 'Informazione miscellanea', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '450', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '450', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '450', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '450', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '450', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '450', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '450', '4', 'Codice relazione', 'Codice relazione', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '450', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '450', '6', 'Collegamento', 'Collegamento', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '450', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '450', '9', '9 (RLIN)', '9 (RLIN)', 0, 0, 4, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '451', 'a', 'Nome geografico', 'Nome geografico', 0, 0, 4, NULL, NULL, '''451z'',''451x'',''451y'',''451v''', 0, 0, '', '', 'GEOGR_NAME'),
		('', '451', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '451', 'g', 'Informazione miscellanea', 'Informazione miscellanea', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '451', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '451', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '451', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '451', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '451', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '451', '4', 'Codice relazione', 'Codice relazione', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '451', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '451', '6', 'Collegamento', 'Collegamento', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '451', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '451', '9', '9 (RLIN)', '9 (RLIN)', 0, 0, 4, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '453', 'a', 'Classification number--Single number or beginning number of span', 'Classification number--Single number or beginning number of span', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '453', 'c', 'Classification element-Ending number of span', 'Classification element-Ending number of span', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '453', 'h', 'Caption hierarchy', 'Caption hierarchy', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '453', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '453', 'j', 'Caption', 'Caption', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '453', 'k', 'Summary number span caption hierarchy', 'Summary number span caption hierarchy', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '453', 't', 'Topic', 'Topic', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '453', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '453', 'y', 'Table identification--Schedule [OBSOLETE]', 'Table identification--Schedule [OBSOLETE]', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '453', 'z', 'Identificazione tavola', 'Identificazione tavola', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '453', '6', 'Collegamento', 'Collegamento', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '453', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '453', '9', '9 (RLIN)', '9 (RLIN)', 0, 0, 4, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '455', 'a', 'Termine di genere/forma', 'Termine di genere/forma', 0, 0, 4, NULL, NULL, '''455v'',''455x'',''455z'',''455y''', 0, 0, '', '', 'GENRE/FORM'),
		('', '455', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '455', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '455', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '455', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '455', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '455', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '455', '4', 'Codice relazione', 'Codice relazione', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '455', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '455', '6', 'Collegamento', 'Collegamento', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '455', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '455', '9', '9 (RLIN)', '9 (RLIN)', 0, 0, 4, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '462', 'a', 'Termine del mezzo di esecuzione', 'Termine del mezzo di esecuzione', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '462', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '462', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '462', '4', 'Codice relazione', 'Codice relazione', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '462', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '462', '6', 'Collegamento', 'Collegamento', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '462', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '480', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '480', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '480', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '480', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '480', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '480', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '480', '4', 'Codice relazione', 'Codice relazione', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '480', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '480', '6', 'Collegamento', 'Collegamento', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '480', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '481', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '481', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '481', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '481', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '481', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '481', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '481', '4', 'Codice relazione', 'Codice relazione', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '481', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '481', '6', 'Collegamento', 'Collegamento', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '481', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '482', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '482', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '482', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '482', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '482', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '482', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '482', '4', 'Codice relazione', 'Codice relazione', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '482', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '482', '6', 'Collegamento', 'Collegamento', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '482', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '485', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '485', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '485', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '485', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '485', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '485', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '485', '4', 'Codice relazione', 'Codice relazione', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '485', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '485', '6', 'Collegamento', 'Collegamento', 0, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '485', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 4, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'a', 'Nome di persona', 'Nome di persona', 0, 0, 5, NULL, NULL, '''500b'',''500c'',''500q'',''500d'',''500t'',''500o'',''500m'',''500r'',''500s'',''500k'',''500n'',''500p'',''500g'',''500l'',''500f'',''500h'',''500x'',''500z'',''500y'',''500v''', 0, 0, '', '', 'PERSO_NAME'),
		('', '500', 'b', 'Numerazione', 'Numerazione', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'c', 'Titoli e altre parole associate al nome', 'Titoli e altre parole associate al nome', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'd', 'Date associate al nome', 'Date associate al nome', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'e', 'Termine di ruolo', 'Termine di ruolo', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'f', 'Data dell\'opera', 'Data dell\'opera', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'g', 'Informazione miscellanea', 'Informazione miscellanea', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'h', 'Supporto', 'Supporto', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'j', 'Qualificazione d\'attribuzione', 'Qualificazione d\'attribuzione', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'k', 'Sottointestazione formale', 'Sottointestazione formale', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'l', 'Lingua dell\'opera', 'Lingua dell\'opera', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'm', 'Mezzo di esecuzione per la musica', 'Mezzo di esecuzione per la musica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'n', 'Numero di parte/sezione dell\'opera', 'Numero di parte/sezione dell\'opera', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'o', 'Formulazione d\'arrangiamento per la musica', 'Formulazione d\'arrangiamento per la musica', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'p', 'Nome di parte/sezione dell\'opera', 'Nome di parte/sezione dell\'opera', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'q', 'Forma completa del nome', 'Forma completa del nome', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'r', 'Tonalità (musica)', 'Tonalità (musica)', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 's', 'Versione', 'Versione', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 't', 'Titolo dell\'opera', 'Titolo dell\'opera', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', '0', 'Numero del record di authority o numero standard', 'Numero del record di authority o numero standard', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', '4', 'Codice relazione', 'Codice relazione', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', '6', 'Collegamento', 'Collegamento', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '500', '9', '9 (RLIN)', '9 (RLIN)', 0, 0, 5, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '510', 'a', 'Nome dell\'ente o nome della giurisdizione come accesso', 'Nome dell\'ente o nome della giurisdizione come accesso', 0, 0, 5, NULL, NULL, '''510b'',''510c'',''510d'',''510t'',''510o'',''510m'',''510r'',''510s'',''510k'',''510n'',''510p'',''510g'',''510l'',''510f'',''510h'',''510x'',''510z'',''510y'',''510v''', 0, 0, '', '', 'CORPO_NAME'),
		('', '510', 'b', 'Unità subordinata', 'Unità subordinata', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'c', 'Luogo del convegno', 'Luogo del convegno', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'd', 'Data del convegno o di firma del trattato', 'Data del convegno o di firma del trattato', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'e', 'Termine di ruolo', 'Termine di ruolo', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'f', 'Data dell\'opera', 'Data dell\'opera', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'g', 'Informazione miscellanea', 'Informazione miscellanea', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'h', 'Supporto', 'Supporto', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'k', 'Sottointestazione formale', 'Sottointestazione formale', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'l', 'Lingua dell\'opera', 'Lingua dell\'opera', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'm', 'Mezzo di esecuzione per la musica', 'Mezzo di esecuzione per la musica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'n', 'Numero di parte/sezione/convegno', 'Numero di parte/sezione/convegno', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'o', 'Formulazione d\'arrangiamento per la musica', 'Formulazione d\'arrangiamento per la musica', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'p', 'Nome di parte/sezione dell\'opera', 'Nome di parte/sezione dell\'opera', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'r', 'Tonalità (musica)', 'Tonalità (musica)', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 's', 'Versione', 'Versione', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 't', 'Titolo dell\'opera', 'Titolo dell\'opera', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', '0', 'Numero del record di authority o numero standard', 'Numero del record di authority o numero standard', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', '4', 'Codice relazione', 'Codice relazione', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', '6', 'Collegamento', 'Collegamento', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '510', '9', '9 (RLIN)', '9 (RLIN)', 0, 0, 5, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '511', 'a', 'Nome del convegno o nome della giurisdizione come accesso', 'Nome del convegno o nome della giurisdizione come accesso', 0, 0, 5, NULL, NULL, '''511e'',''511c'',''511d'',''511t'',''511s'',''511k'',''511n'',''511p'',''511g'',''511l'',''511f'',''511h'',''511x'',''511z'',''511y'',''511v''', 0, 0, '', '', 'MEETI_NAME'),
		('', '511', 'c', 'Luogo del convegno', 'Luogo del convegno', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 'd', 'Data del convegno', 'Data del convegno', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 'e', 'Unità subordinata', 'Unità subordinata', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 'f', 'Data dell\'opera', 'Data dell\'opera', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 'g', 'Informazione miscellanea', 'Informazione miscellanea', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 'h', 'Supporto', 'Supporto', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 'j', 'Termine di ruolo', 'Termine di ruolo', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 'k', 'Sottointestazione formale', 'Sottointestazione formale', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 'l', 'Lingua dell\'opera', 'Lingua dell\'opera', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 'n', 'Numero di parte/sezione/convegno', 'Numero di parte/sezione/convegno', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 'p', 'Nome di parte/sezione dell\'opera', 'Nome di parte/sezione dell\'opera', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 'q', 'Nome del convegno dopo accesso di nome della giurisdizione', 'Nome del convegno dopo accesso di nome della giurisdizione', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 's', 'Versione', 'Versione', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 't', 'Titolo dell\'opera', 'Titolo dell\'opera', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', '0', 'Numero del record di authority o numero standard', 'Numero del record di authority o numero standard', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', '4', 'Codice relazione', 'Codice relazione', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', '6', 'Collegamento', 'Collegamento', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '511', '9', '9 (RLIN)', '9 (RLIN)', 0, 0, 5, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '530', 'a', 'Titolo uniforme', 'Titolo uniforme', 0, 0, 5, NULL, NULL, '''530o'',''530m'',''530r'',''530s'',''530d'',''530k'',''530n'',''530p'',''530g'',''530l'',''530f'',''530h'',''530t'',''530x'',''530z'',''530y'',''530v''', 0, 0, '', '', 'UNIF_TITLE'),
		('', '530', 'd', 'Data di firma del trattato', 'Data di firma del trattato', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 'f', 'Data dell\'opera', 'Data dell\'opera', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 'g', 'Informazione miscellanea', 'Informazione miscellanea', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 'h', 'Supporto', 'Supporto', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 'k', 'Sottointestazione formale', 'Sottointestazione formale', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 'l', 'Lingua dell\'opera', 'Lingua dell\'opera', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 'm', 'Mezzo di esecuzione per la musica', 'Mezzo di esecuzione per la musica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 'n', 'Numero di parte/sezione dell\'opera', 'Numero di parte/sezione dell\'opera', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 'o', 'Formulazione d\'arrangiamento per la musica', 'Formulazione d\'arrangiamento per la musica', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 'p', 'Nome di parte/sezione dell\'opera', 'Nome di parte/sezione dell\'opera', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 'r', 'Tonalità (musica)', 'Tonalità (musica)', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 's', 'Versione', 'Versione', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 't', 'Titolo dell\'opera', 'Titolo dell\'opera', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', '0', 'Numero del record di authority o numero standard', 'Numero del record di authority o numero standard', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', '4', 'Codice relazione', 'Codice relazione', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', '6', 'Collegamento', 'Collegamento', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '530', '9', '9 (RLIN)', '9 (RLIN)', 0, 0, 5, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '548', 'a', 'Termine cronologico', 'Termine cronologico', 0, 0, 5, NULL, NULL, '''548y'',''548x'',''548z'',''548v''', 0, 0, '', '', 'CHRON_TERM'),
		('', '548', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '548', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '548', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '548', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '548', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '548', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '548', '0', 'Numero di controllo della registrazione', 'Numero di controllo della registrazione', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '548', '4', 'Codice relazione', 'Codice relazione', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '548', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '548', '6', 'Collegamento', 'Collegamento', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '548', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '548', '9', '9 (RLIN)', '9 (RLIN)', 0, 0, 5, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '550', 'a', 'Termine topico o nome geografico come accesso', 'Termine topico o nome geografico come accesso', 0, 0, 5, NULL, NULL, '''550x'',''550z'',''550y'',''550v''', 0, 0, '', '', 'TOPIC_TERM'),
		('', '550', 'b', 'Termine topico che segue il nome geografico come accesso', 'Termine topico che segue il nome geografico come accesso', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '550', 'g', 'Informazione miscellanea', 'Informazione miscellanea', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '550', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '550', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '550', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '550', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '550', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '550', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '550', '0', 'Numero del record di authority o numero standard', 'Numero del record di authority o numero standard', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '550', '4', 'Codice relazione', 'Codice relazione', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '550', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '550', '6', 'Collegamento', 'Collegamento', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '550', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '550', '9', '9 (RLIN)', '9 (RLIN)', 0, 0, 5, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '551', 'a', 'Nome geografico', 'Nome geografico', 0, 0, 5, NULL, NULL, '''551z'',''551x'',''551y'',''551v''', 0, 0, '', '', 'GEOGR_NAME'),
		('', '551', 'g', 'Informazione miscellanea', 'Informazione miscellanea', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '551', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '551', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '551', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '551', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '551', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '551', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '551', '0', 'Numero del record di authority o numero standard', 'Numero del record di authority o numero standard', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '551', '4', 'Codice relazione', 'Codice relazione', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '551', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '551', '6', 'Collegamento', 'Collegamento', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '551', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '551', '9', '9 (RLIN)', '9 (RLIN)', 0, 0, 5, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '553', 'a', 'Classification number--Single number or beginning number of span', 'Classification number--Single number or beginning number of span', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '553', 'c', 'Numero di classificazione – fine intervallo', 'Numero di classificazione – fine intervallo', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '553', 'h', 'Caption hierarchy', 'Caption hierarchy', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '553', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '553', 'j', 'Caption', 'Caption', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '553', 'k', 'Summary number span caption hierarchy', 'Summary number span caption hierarchy', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '553', 't', 'Topic', 'Topic', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '553', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '553', 'y', 'Table identification--Schedule [OBSOLETE]', 'Table identification--Schedule [OBSOLETE]', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '553', 'z', 'Identificazione tavola', 'Identificazione tavola', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '553', '6', 'Collegamento', 'Collegamento', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '553', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '553', '9', '9 (RLIN)', '9 (RLIN)', 0, 0, 5, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '555', 'a', 'Termine di genere/forma', 'Termine di genere/forma', 0, 0, 5, NULL, NULL, '''555v'',''555x'',''555z'',''555y''', 0, 0, '', '', 'GENRE/FORM'),
		('', '555', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '555', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '555', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '555', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '555', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '555', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '555', '0', 'Numero di controllo della registrazione', 'Numero di controllo della registrazione', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '555', '4', 'Codice relazione', 'Codice relazione', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '555', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '555', '6', 'Collegamento', 'Collegamento', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '555', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '555', '9', '9 (RLIN)', '9 (RLIN)', 0, 0, 5, NULL, NULL, NULL, 0, 1, '', '', ''),
		('', '562', 'a', 'Termine del mezzo di esecuzione', 'Termine del mezzo di esecuzione', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '562', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '562', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '562', '0', 'Numero del record di authority o numero standard', 'Numero del record di authority o numero standard', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '562', '4', 'Codice relazione', 'Codice relazione', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '562', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '562', '6', 'Collegamento', 'Collegamento', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '562', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '580', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '580', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '580', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '580', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '580', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '580', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '580', '0', 'Numero di controllo della registrazione', 'Numero di controllo della registrazione', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '580', '4', 'Codice relazione', 'Codice relazione', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '580', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '580', '6', 'Collegamento', 'Collegamento', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '580', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '581', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '581', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '581', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '581', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '581', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '581', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '581', '0', 'Numero di controllo della registrazione', 'Numero di controllo della registrazione', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '581', '4', 'Codice relazione', 'Codice relazione', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '581', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '581', '6', 'Collegamento', 'Collegamento', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '581', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '582', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '582', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '582', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '582', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '582', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '582', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '582', '0', 'Numero di controllo della registrazione', 'Numero di controllo della registrazione', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '582', '4', 'Codice relazione', 'Codice relazione', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '582', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '582', '6', 'Collegamento', 'Collegamento', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '582', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '585', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '585', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '585', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '585', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '585', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '585', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '585', '0', 'Numero di controllo della registrazione', 'Numero di controllo della registrazione', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '585', '4', 'Codice relazione', 'Codice relazione', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '585', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '585', '6', 'Collegamento', 'Collegamento', 0, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '585', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 5, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '640', 'a', 'Date della pubblicazione e/o designazione sequenziale', 'Date della pubblicazione e/o designazione sequenziale', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '640', 'z', 'Fonte dell\'informazione', 'Fonte dell\'informazione', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '640', '6', 'Collegamento', 'Collegamento', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '640', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '641', 'a', 'Nota su particolarità della numerazione', 'Nota su particolarità della numerazione', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '641', 'z', 'Fonte dell\'informazione', 'Fonte dell\'informazione', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '641', '6', 'Collegamento', 'Collegamento', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '641', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '642', 'a', 'Esempio di numerazione della serie', 'Esempio di numerazione della serie', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '642', 'd', 'Volumi/date a cui si applica l\'esempio di numerazione della serie', 'Volumi/date a cui si applica l\'esempio di numerazione della serie', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '642', '5', 'Istituzione/copia a cui si applica il campo', 'Istituzione/copia a cui si applica il campo', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '642', '6', 'Collegamento', 'Collegamento', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '642', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '643', 'a', 'Luogo', 'Luogo', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '643', 'b', 'Editore/ente responsabile', 'Editore/ente responsabile', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '643', 'd', 'Volumi/date a cui si applica l\'esempio di numerazione della serie', 'Volumi/date a cui si applica l\'esempio di numerazione della serie', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '643', '6', 'Collegamento', 'Collegamento', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '643', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '644', 'a', 'Procedura di analisi della serie', 'Procedura di analisi della serie', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '644', 'b', 'Eccezioni alla procedura di analisi', 'Eccezioni alla procedura di analisi', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '644', 'd', 'Volumi/date a cui si applica la procedura di analisi', 'Volumi/date a cui si applica la procedura di analisi', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '644', '5', 'Istituzione/copia a cui si applica il campo', 'Istituzione/copia a cui si applica il campo', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '644', '6', 'Collegamento', 'Collegamento', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '644', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '645', 'a', 'Procedura di tracciato della serie', 'Procedura di tracciato della serie', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '645', 'd', 'Volumi/date a cui si applica la procedura di tracciato', 'Volumi/date a cui si applica la procedura di tracciato', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '645', '5', 'Istituzione/copia a cui si applica il campo', 'Istituzione/copia a cui si applica il campo', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '645', '6', 'Collegamento', 'Collegamento', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '645', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '646', 'a', 'Procedura di classificazione della serie', 'Procedura di classificazione della serie', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '646', 'd', 'Volumi/date a cui si applica la procedura di classificazione', 'Volumi/date a cui si applica la procedura di classificazione', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '646', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '646', '6', 'Collegamento', 'Collegamento', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '646', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '663', 'a', 'Spiegazione', 'Spiegazione', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '663', 'b', 'Luogo di pubblicazione, distribuzione, ecc.', 'Luogo di pubblicazione, distribuzione, ecc.', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '663', 't', 'Titolo a cui si rinvia', 'Titolo a cui si rinvia', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '663', '6', 'Collegamento', 'Collegamento', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '663', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '664', 'a', 'Spiegazione', 'Spiegazione', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '664', 'b', 'Luogo di pubblicazione, distribuzione, ecc.', 'Luogo di pubblicazione, distribuzione, ecc.', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '664', 't', 'Titolo a cui si rinvia', 'Titolo a cui si rinvia', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '664', '6', 'Collegamento', 'Collegamento', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '664', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '665', 'a', 'Rinvio storico', 'Rinvio storico', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '665', '6', 'Collegamento', 'Collegamento', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '665', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '666', 'a', 'Rinvio esplicativo generale', 'Rinvio esplicativo generale', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '666', '6', 'Collegamento', 'Collegamento', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '666', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '667', 'a', 'Nota generale non pubblica', 'Nota generale non pubblica', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '667', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '667', '6', 'Collegamento', 'Collegamento', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '667', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '668', 'a', 'Characters in nonroman alphabet', 'Characters in nonroman alphabet', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '670', 'a', 'Citazione della fonte', 'Citazione della fonte', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '670', 'b', 'Informazione trovata', 'Informazione trovata', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '670', 'u', 'URI (Uniform Resource Identifier)', 'URI (Uniform Resource Identifier)', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '670', 'w', 'Numero di controllo bibliografico', 'Numero di controllo bibliografico', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '670', '6', 'Collegamento', 'Collegamento', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '670', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '671', 'a', 'Citazione', 'Citazione', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '671', '6', 'Collegamento', 'Collegamento', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '672', 'a', 'Titolo', 'Titolo', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '672', 'b', 'Complemento del titolo', 'Complemento del titolo', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '672', 'f', 'Data', 'Data', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '672', 'w', 'Numero di controllo bibliografico', 'Numero di controllo bibliografico', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '672', '0', 'Numero del record di authority o numero standard', 'Numero del record di authority o numero standard', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '672', '6', 'Collegamento', 'Collegamento', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '672', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '673', 'a', 'Titolo', 'Titolo', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '673', 'b', 'Complemento del titolo', 'Complemento del titolo', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '673', 'f', 'Data', 'Data', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '673', 'w', 'Numero di controllo bibliografico', 'Numero di controllo bibliografico', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '673', '0', 'Numero del record di authority o numero standard', 'Numero del record di authority o numero standard', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '673', '6', 'Collegamento', 'Collegamento', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '673', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '675', 'a', 'Citazione della fonte', 'Citazione della fonte', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '675', '6', 'Collegamento', 'Collegamento', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '675', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '676', 'a', 'Regola della numerazione e ulteriori informazioni', 'Regola della numerazione e ulteriori informazioni', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '676', '6', 'Collegamento', 'Collegamento', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '678', 'a', 'Nota biografica o storica', 'Nota biografica o storica', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '678', 'b', 'Ampliamento', 'Ampliamento', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '678', 'u', 'URI (Uniform Resource Identifier)', 'URI (Uniform Resource Identifier)', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '678', '6', 'Collegamento', 'Collegamento', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '678', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '680', 'a', 'Intestazione o termine di suddivisione', 'Intestazione o termine di suddivisione', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '680', 'c', 'Numero di classificazione--Numero finale di un intervallo [Formato per classificazione]', 'Numero di classificazione--Numero finale di un intervallo [Formato per classificazione]', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '680', 'i', 'Spiegazione', 'Spiegazione', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '680', 'z', 'Identificazione tavola', 'Identificazione tavola', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '680', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '680', '6', 'Collegamento', 'Collegamento', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '680', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '681', 'a', 'Intestazione di soggetto o termine di suddivisione', 'Intestazione di soggetto o termine di suddivisione', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '681', 'c', 'Numero di classificazione--Numero finale di un intervallo [Formato per classificazione]', 'Numero di classificazione--Numero finale di un intervallo [Formato per classificazione]', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '681', 'i', 'Spiegazione', 'Spiegazione', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '681', 'z', 'Identificazione tavola', 'Identificazione tavola', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '681', '6', 'Collegamento', 'Collegamento', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '681', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '682', 'a', 'Intestazione di sostituzione', 'Intestazione di sostituzione', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '682', 'i', 'Spiegazione', 'Spiegazione', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '682', '0', 'Numero di controllo della registrazione di autorità di sostituzione', 'Numero di controllo della registrazione di autorità di sostituzione', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '682', '6', 'Collegamento', 'Collegamento', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '682', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '683', 'a', 'Classification number--Single number or beginning number of span', 'Classification number--Single number or beginning number of span', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '683', 'c', 'Numero di classificazione – fine intervallo', 'Numero di classificazione – fine intervallo', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '683', 'i', 'Spiegazione', 'Spiegazione', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '683', 'p', 'Campo classificazione corrispondente', 'Campo classificazione corrispondente', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '683', 'y', 'Table identification--Schedule [OBSOLETE]', 'Table identification--Schedule [OBSOLETE]', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '683', 'z', 'Identificazione tavola', 'Identificazione tavola', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '683', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '683', '6', 'Collegamento', 'Collegamento', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '683', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '684', 'a', 'Classification number--Single number or beginning number of span', 'Classification number--Single number or beginning number of span', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '684', 'c', 'Numero di classificazione – fine intervallo', 'Numero di classificazione – fine intervallo', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '684', 'i', 'Spiegazione', 'Spiegazione', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '684', 'j', 'Caption', 'Caption', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '684', 'z', 'Identificazione tavola', 'Identificazione tavola', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '684', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '684', '6', 'Collegamento', 'Collegamento', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '684', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '685', 'a', 'Source consulted and information found', 'Source consulted and information found', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '685', '6', 'Collegamento', 'Collegamento', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '686', 'a', 'Source consulted and information found', 'Source consulted and information found', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '686', '6', 'Collegamento', 'Collegamento', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '687', 'a', 'Source consulted and information found', 'Source consulted and information found', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '687', '6', 'Collegamento', 'Collegamento', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '688', 'a', 'Nota sulla storia dell’applicazione', 'Nota sulla storia dell’applicazione', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '688', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '688', '6', 'Collegamento', 'Collegamento', 0, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '688', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 6, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'a', 'Nome di persona', 'Nome di persona', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'b', 'Numerazione', 'Numerazione', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'c', 'Titoli e altre parole associate al nome', 'Titoli e altre parole associate al nome', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'd', 'Date associate al nome', 'Date associate al nome', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'e', 'Termine di ruolo', 'Termine di ruolo', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'f', 'Data dell\'opera', 'Data dell\'opera', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'g', 'Informazione miscellanea', 'Informazione miscellanea', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'h', 'Supporto', 'Supporto', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'j', 'Qualificazione d\'attribuzione', 'Qualificazione d\'attribuzione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'k', 'Sottointestazione formale', 'Sottointestazione formale', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'l', 'Lingua dell\'opera', 'Lingua dell\'opera', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'm', 'Mezzo di esecuzione per la musica', 'Mezzo di esecuzione per la musica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'n', 'Numero di parte/sezione dell\'opera', 'Numero di parte/sezione dell\'opera', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'o', 'Formulazione d\'arrangiamento per la musica', 'Formulazione d\'arrangiamento per la musica', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'p', 'Nome di parte/sezione dell\'opera', 'Nome di parte/sezione dell\'opera', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'q', 'Forma completa del nome', 'Forma completa del nome', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'r', 'Tonalità (musica)', 'Tonalità (musica)', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 's', 'Versione', 'Versione', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 't', 'Titolo dell\'opera', 'Titolo dell\'opera', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', '0', 'Numero del record di authority o numero standard', 'Numero del record di authority o numero standard', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', '2', 'Fonte dell\'intestazione o del termine', 'Fonte dell\'intestazione o del termine', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', '3', 'Materiale specificato [Formato per classificazione]', 'Materiale specificato [Formato per classificazione]', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', '4', 'Codice relazione', 'Codice relazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', '6', 'Collegamento', 'Collegamento', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '700', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'a', 'Nome dell\'ente o nome della giurisdizione come accesso', 'Nome dell\'ente o nome della giurisdizione come accesso', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'b', 'Unità subordinata', 'Unità subordinata', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'c', 'Luogo del convegno', 'Luogo del convegno', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'd', 'Data del convegno o di firma del trattato', 'Data del convegno o di firma del trattato', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'e', 'Termine di ruolo', 'Termine di ruolo', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'f', 'Data dell\'opera', 'Data dell\'opera', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'g', 'Informazione miscellanea', 'Informazione miscellanea', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'h', 'Supporto', 'Supporto', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'k', 'Sottointestazione formale', 'Sottointestazione formale', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'l', 'Lingua dell\'opera', 'Lingua dell\'opera', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'm', 'Mezzo di esecuzione per la musica', 'Mezzo di esecuzione per la musica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'n', 'Numero di parte/sezione/convegno', 'Numero di parte/sezione/convegno', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'o', 'Formulazione d\'arrangiamento per la musica', 'Formulazione d\'arrangiamento per la musica', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'p', 'Nome di parte/sezione dell\'opera', 'Nome di parte/sezione dell\'opera', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'r', 'Tonalità (musica)', 'Tonalità (musica)', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 's', 'Versione', 'Versione', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 't', 'Titolo dell\'opera', 'Titolo dell\'opera', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', '0', 'Numero del record di authority o numero standard', 'Numero del record di authority o numero standard', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', '2', 'Fonte dell\'intestazione o del termine', 'Fonte dell\'intestazione o del termine', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', '3', 'Materiale specificato [Formato per classificazione]', 'Materiale specificato [Formato per classificazione]', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', '4', 'Codice relazione', 'Codice relazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', '6', 'Collegamento', 'Collegamento', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '710', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'a', 'Nome del convegno o nome della giurisdizione come accesso', 'Nome del convegno o nome della giurisdizione come accesso', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'c', 'Luogo del convegno', 'Luogo del convegno', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'd', 'Data del convegno', 'Data del convegno', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'e', 'Unità subordinata', 'Unità subordinata', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'f', 'Data dell\'opera', 'Data dell\'opera', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'g', 'Informazione miscellanea', 'Informazione miscellanea', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'h', 'Supporto', 'Supporto', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'j', 'Termine di ruolo', 'Termine di ruolo', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'k', 'Sottointestazione formale', 'Sottointestazione formale', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'l', 'Lingua dell\'opera', 'Lingua dell\'opera', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'n', 'Numero di parte/sezione/convegno', 'Numero di parte/sezione/convegno', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'p', 'Nome di parte/sezione dell\'opera', 'Nome di parte/sezione dell\'opera', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'q', 'Nome del convegno dopo accesso di nome della giurisdizione', 'Nome del convegno dopo accesso di nome della giurisdizione', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 's', 'Versione', 'Versione', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 't', 'Titolo dell\'opera', 'Titolo dell\'opera', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', '0', 'Numero del record di authority o numero standard', 'Numero del record di authority o numero standard', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', '2', 'Fonte dell\'intestazione o del termine', 'Fonte dell\'intestazione o del termine', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', '3', 'Materiale specificato', 'Materiale specificato', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', '4', 'Codice relazione', 'Codice relazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', '6', 'Collegamento', 'Collegamento', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '711', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'a', 'Titolo uniforme', 'Titolo uniforme', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'd', 'Data di firma del trattato', 'Data di firma del trattato', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'f', 'Data dell\'opera', 'Data dell\'opera', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'g', 'Informazione miscellanea', 'Informazione miscellanea', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'h', 'Supporto', 'Supporto', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'k', 'Sottointestazione formale', 'Sottointestazione formale', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'l', 'Lingua dell\'opera', 'Lingua dell\'opera', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'm', 'Mezzo di esecuzione per la musica', 'Mezzo di esecuzione per la musica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'n', 'Numero di parte/sezione dell\'opera', 'Numero di parte/sezione dell\'opera', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'o', 'Formulazione d\'arrangiamento per la musica', 'Formulazione d\'arrangiamento per la musica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'p', 'Nome di parte/sezione dell\'opera', 'Nome di parte/sezione dell\'opera', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'r', 'Tonalità (musica)', 'Tonalità (musica)', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 's', 'Versione', 'Versione', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 't', 'Titolo dell\'opera', 'Titolo dell\'opera', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', '0', 'Numero del record di authority o numero standard', 'Numero del record di authority o numero standard', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', '2', 'Fonte dell\'intestazione o del termine', 'Fonte dell\'intestazione o del termine', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', '3', 'Materiale specificato [Formato per classificazione]', 'Materiale specificato [Formato per classificazione]', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', '4', 'Codice relazione', 'Codice relazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', '6', 'Collegamento', 'Collegamento', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '730', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '748', 'a', 'Termine cronologico', 'Termine cronologico', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '748', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '748', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '748', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '748', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '748', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '748', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '748', '0', 'Numero del record di authority o numero standard', 'Numero del record di authority o numero standard', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '748', '2', 'Fonte dell\'intestazione o del termine', 'Fonte dell\'intestazione o del termine', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '748', '3', 'Materiale specificato [Formato per classificazione]', 'Materiale specificato [Formato per classificazione]', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '748', '4', 'Codice relazione', 'Codice relazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '748', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '748', '6', 'Collegamento', 'Collegamento', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '748', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', 'a', 'Termine topico o nome geografico come accesso', 'Termine topico o nome geografico come accesso', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', 'b', 'Termine topico che segue il nome geografico come accesso', 'Termine topico che segue il nome geografico come accesso', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', 'c', 'Luogo dell\'evento', 'Luogo dell\'evento', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', 'd', 'Data d\'attività', 'Data d\'attività', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', 'g', 'Informazione miscellanea', 'Informazione miscellanea', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', '0', 'Numero del record di authority o numero standard', 'Numero del record di authority o numero standard', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', '2', 'Fonte dell\'intestazione o del termine', 'Fonte dell\'intestazione o del termine', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', '3', 'Materiale specificato [Formato per classificazione]', 'Materiale specificato [Formato per classificazione]', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', '4', 'Codice relazione', 'Codice relazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', '6', 'Collegamento', 'Collegamento', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '750', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '751', 'a', 'Nome geografico', 'Nome geografico', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '751', 'g', 'Informazione miscellanea', 'Informazione miscellanea', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '751', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '751', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '751', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '751', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '751', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '751', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '751', '0', 'Numero del record di authority o numero standard', 'Numero del record di authority o numero standard', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '751', '2', 'Fonte dell\'intestazione o del termine', 'Fonte dell\'intestazione o del termine', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '751', '3', 'Materiale specificato [Formato per classificazione]', 'Materiale specificato [Formato per classificazione]', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '751', '4', 'Codice relazione', 'Codice relazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '751', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '751', '6', 'Collegamento', 'Collegamento', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '751', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '753', 'a', 'Index term', 'Index term', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '753', 'b', 'Index term--Succeeding level', 'Index term--Succeeding level', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '753', 'd', 'Index term referred from', 'Index term referred from', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '753', 'e', 'Example class number', 'Example class number', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '753', 'i', 'Spiegazione', 'Spiegazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '753', 's', 'See also reference term', 'See also reference term', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '753', 't', 'See also reference term--Succeeding level', 'See also reference term--Succeeding level', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '753', 'u', 'Use reference term', 'Use reference term', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '753', 'v', 'Use reference term--Succeeding level', 'Use reference term--Succeeding level', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '753', '6', 'Collegamento', 'Collegamento', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '753', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '754', 'a', 'Termine principale', 'Termine principale', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '754', 'b', 'Termine non principale', 'Termine non principale', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '754', 'c', 'Designazione di faccetta/gerarchia', 'Designazione di faccetta/gerarchia', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '754', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '754', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '754', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '754', '2', 'Fonte dell\'intestazione o del termine', 'Fonte dell\'intestazione o del termine', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '754', '3', 'Materiale specificato', 'Materiale specificato', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '754', '6', 'Collegamento', 'Collegamento', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '754', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '755', 'a', 'Termine di genere/forma come accesso', 'Termine di genere/forma come accesso', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '755', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '755', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '755', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '755', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '755', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '755', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '755', '0', 'Numero del record di authority o numero standard', 'Numero del record di authority o numero standard', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '755', '2', 'Fonte dell\'intestazione o del termine', 'Fonte dell\'intestazione o del termine', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '755', '4', 'Codice relazione', 'Codice relazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '755', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '755', '6', 'Collegamento', 'Collegamento', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '755', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '761', 'a', 'Numero dove ci sono le istruzioni -- Numero singolo o inizio di un intervallo', 'Numero dove ci sono le istruzioni -- Numero singolo o inizio di un intervallo', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '761', 'b', 'Numero di base', 'Numero di base', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '761', 'c', 'Numero di classificazione – fine intervallo', 'Numero di classificazione – fine intervallo', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '761', 'd', 'Diviso come un numero', 'Diviso come un numero', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '761', 'e', 'Example class number', 'Example class number', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '761', 'f', 'Designatore faccetta', 'Designatore faccetta', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '761', 'i', 'Spiegazione', 'Spiegazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '761', 'n', 'Esempio di numero di classificazione negativo', 'Esempio di numero di classificazione negativo', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '761', 'r', 'Numero radice', 'Numero radice', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '761', 'x', 'Numero di altro sistema di classificazione', 'Numero di altro sistema di classificazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '761', 'z', 'Identificazione tavola', 'Identificazione tavola', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '761', '6', 'Collegamento', 'Collegamento', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '761', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '762', 'a', 'Termine del mezzo di esecuzione come accesso', 'Termine del mezzo di esecuzione come accesso', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '762', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '762', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '762', 'z', 'Tavola dei numeri', 'Tavola dei numeri', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '762', '0', 'Numero del record di authority o numero standard', 'Numero del record di authority o numero standard', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '762', '2', 'Fonte dell\'intestazione o del termine', 'Fonte dell\'intestazione o del termine', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '762', '4', 'Codice relazione', 'Codice relazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '762', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '762', '6', 'Collegamento', 'Collegamento', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '762', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', 'a', 'Classification number--Single number or beginning number of span', 'Classification number--Single number or beginning number of span', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', 'b', 'Numero di base', 'Numero di base', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', 'c', 'Numero di classificazione – fine intervallo', 'Numero di classificazione – fine intervallo', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', 'd', 'Diviso come un numero', 'Diviso come un numero', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', 'e', 'Example class number', 'Example class number', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', 'h', 'Caption hierarchy', 'Caption hierarchy', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', 'i', 'Spiegazione', 'Spiegazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', 'j', 'Caption', 'Caption', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', 'k', 'Summary number span caption hierarchy', 'Summary number span caption hierarchy', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', 'm', 'Nota al manuale', 'Nota al manuale', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', 'n', 'Numero dove sono le istruzioni', 'Numero dove sono le istruzioni', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', 'p', 'Campo classificazione corrispondente', 'Campo classificazione corrispondente', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', 'r', 'Numero radice', 'Numero radice', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', 's', 'Vedi riinvio', 'Vedi riinvio', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', 'x', 'Numero di altro sistema di classificazione', 'Numero di altro sistema di classificazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', 'z', 'Identificazione tavola', 'Identificazione tavola', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', '6', 'Collegamento', 'Collegamento', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '763', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '764', 'a', 'Numero regola', 'Numero regola', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '764', 'i', 'Spiegazione', 'Spiegazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '764', '6', 'Collegamento', 'Collegamento', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '764', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '765', 'a', 'Number where instructions are found--Single number or beginning number of span', 'Number where instructions are found--Single number or beginning number of span', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '765', 'b', 'Numero di base', 'Numero di base', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '765', 'c', 'Number where instructions are found--Ending number of span', 'Number where instructions are found--Ending number of span', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '765', 'f', 'Designatore faccetta', 'Designatore faccetta', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '765', 'r', 'Numero radice', 'Numero radice', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '765', 's', 'Numeri aggiunti per costruire la stringa', 'Numeri aggiunti per costruire la stringa', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '765', 't', 'Numeri aggiunti da partizione interna o tavola aggiunta', 'Numeri aggiunti da partizione interna o tavola aggiunta', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '765', 'u', 'Numero da analizzare', 'Numero da analizzare', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '765', 'v', 'Numeri aggiunti da partizione interna o tavola aggiunta dove trovate le istruzioni', 'Numeri aggiunti da partizione interna o tavola aggiunta dove trovate le istruzioni', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '765', 'w', 'Table identification--Internal subarrangement or add table', 'Table identification--Internal subarrangement or add table', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '765', 'y', 'Table identification--Schedule [OBSOLETE]', 'Table identification--Schedule [OBSOLETE]', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '765', 'z', 'Identificazione tavola', 'Identificazione tavola', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '765', '6', 'Collegamento', 'Collegamento', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '765', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '766', 'a', 'Secondary table of applicability', 'Secondary table of applicability', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '766', 'y', 'Type of division', 'Type of division', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '766', '6', 'Collegamento', 'Collegamento', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '766', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '768', 'a', 'Classification number--Single number or beginning number of span', 'Classification number--Single number or beginning number of span', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '768', 'c', 'Numero di classificazione – fine intervallo', 'Numero di classificazione – fine intervallo', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '768', 'e', 'Example class number', 'Example class number', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '768', 'i', 'Spiegazione', 'Spiegazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '768', 'j', 'Caption', 'Caption', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '768', 'n', 'Esempio di numero di classificazione negativo', 'Esempio di numero di classificazione negativo', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '768', 't', 'Argomento usato come esempio di istruzione per cit. e ordine di pref.', 'Argomento usato come esempio di istruzione per cit. e ordine di pref.', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '768', 'x', 'Eccezione alla tavola di preferenza', 'Eccezione alla tavola di preferenza', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '768', 'z', 'Tavola identificativa -- tavola dei numeri', 'Tavola identificativa -- tavola dei numeri', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '768', '6', 'Collegamento', 'Collegamento', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '768', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '780', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '780', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '780', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '780', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '780', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '780', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '780', '0', 'Numero del record di authority o numero standard', 'Numero del record di authority o numero standard', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '780', '2', 'Fonte dell\'intestazione o del termine', 'Fonte dell\'intestazione o del termine', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '780', '4', 'Codice relazione', 'Codice relazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '780', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '780', '6', 'Collegamento', 'Collegamento', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '780', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '781', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '781', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '781', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '781', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '781', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '781', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '781', '0', 'Numero del record di authority o numero standard', 'Numero del record di authority o numero standard', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '781', '2', 'Fonte dell\'intestazione o del termine', 'Fonte dell\'intestazione o del termine', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '781', '4', 'Codice relazione', 'Codice relazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '781', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '781', '6', 'Collegamento', 'Collegamento', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '781', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '782', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '782', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '782', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '782', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '782', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '782', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '782', '0', 'Numero del record di authority o numero standard', 'Numero del record di authority o numero standard', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '782', '2', 'Fonte dell\'intestazione o del termine', 'Fonte dell\'intestazione o del termine', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '782', '4', 'Codice relazione', 'Codice relazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '782', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '782', '6', 'Collegamento', 'Collegamento', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '782', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '785', 'i', 'Informazione sulla relazione', 'Informazione sulla relazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '785', 'v', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '785', 'w', 'Sottocampo di controllo', 'Sottocampo di controllo', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '785', 'x', 'Suddivisione generale', 'Suddivisione generale', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '785', 'y', 'Suddivisione cronologica', 'Suddivisione cronologica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '785', 'z', 'Suddivisione geografica', 'Suddivisione geografica', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '785', '0', 'Numero del record di authority o numero standard', 'Numero del record di authority o numero standard', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '785', '2', 'Fonte dell\'intestazione o del termine', 'Fonte dell\'intestazione o del termine', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '785', '4', 'Codice relazione', 'Codice relazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '785', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '785', '6', 'Collegamento', 'Collegamento', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '785', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '788', 'a', 'Luogo di pubblicazione, distribuzione, ecc.', 'Luogo di pubblicazione, distribuzione, ecc.', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '788', 'i', 'Spiegazione', 'Spiegazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '788', '0', 'Numero di controllo della registrazione', 'Numero di controllo della registrazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '788', '2', 'Fonte dell\'intestazione o del termine', 'Fonte dell\'intestazione o del termine', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '788', '4', 'Codice relazione', 'Codice relazione', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '788', '5', 'Istituzione a cui si applica il campo', 'Istituzione a cui si applica il campo', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '788', '6', 'Collegamento', 'Collegamento', 0, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '788', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 7, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'a', 'Nome dell\'host', 'Nome dell\'host', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'b', 'Numero di accesso', 'Numero di accesso', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'c', 'Informazione sulla compressione', 'Informazione sulla compressione', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'd', 'Percorso della risorsa', 'Percorso della risorsa', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'f', 'Nome elettronico', 'Nome elettronico', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'h', 'Processore della richiesta (Username)', 'Processore della richiesta (Username)', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'i', 'Istruzioni', 'Istruzioni', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'j', 'Bit per secondo', 'Bit per secondo', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'k', 'Password', 'Password', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'l', 'Logon', 'Logon', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'm', 'Contatto per assistenza all\'accesso', 'Contatto per assistenza all\'accesso', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'n', 'Nome della localizzazione dell\'host', 'Nome della localizzazione dell\'host', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'o', 'Sistema operativo', 'Sistema operativo', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'p', 'Porta', 'Porta', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'q', 'Tipo di formato elettronico', 'Tipo di formato elettronico', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'r', 'Parametri', 'Parametri', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 's', 'Dimensione file', 'Dimensione file', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 't', 'Emulazione del terminale', 'Emulazione del terminale', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'u', 'URI (Uniform Resource Identifier)', 'URI (Uniform Resource Identifier)', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'v', 'Orario di accesso', 'Orario di accesso', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'w', 'Numero di controllo della registrazione', 'Numero di controllo della registrazione', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'x', 'Nota non pubblic', 'Nota non pubblic', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'y', 'Testo di collegamento', 'Testo di collegamento', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', 'z', 'Nota pubblica', 'Nota pubblica', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', '2', 'Metodo di accesso', 'Metodo di accesso', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', '3', 'Materiale specificato', 'Materiale specificato', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', '6', 'Collegamento', 'Collegamento', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '856', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'a', 'a', 'a', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'b', 'b', 'b', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'c', 'c', 'c', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'd', 'd', 'd', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'e', 'e', 'e', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'f', 'f', 'f', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'g', 'g', 'g', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'h', 'h', 'h', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'i', 'i', 'i', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'j', 'j', 'j', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'k', 'k', 'k', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'l', 'l', 'l', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'm', 'm', 'm', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'n', 'n', 'n', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'o', 'o', 'o', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'p', 'p', 'p', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'q', 'q', 'q', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'r', 'r', 'r', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 's', 's', 's', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 't', 't', 't', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'u', 'u', 'u', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'v', 'v', 'v', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'w', 'w', 'w', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'x', 'x', 'x', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'y', 'y', 'y', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', 'z', 'z', 'z', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', '0', '0', '0', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', '1', '1', '1', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', '2', '2', '2', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', '3', '3', '3', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', '4', '4', '4', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', '5', '5', '5', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', '6', 'Collegamento', 'Collegamento', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', '7', '7', '7', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', '8', '8', '8', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '880', '9', '9', '9', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '883', 'a', 'Processo di generazione', 'Processo di generazione', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '883', 'c', 'Attendibilità dei metadati (valore)', 'Attendibilità dei metadati (valore)', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '883', 'd', 'Data di generazione', 'Data di generazione', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '883', 'q', 'Agenzia di generazione', 'Agenzia di generazione', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '883', 'x', 'Fine data validità', 'Fine data validità', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '883', 'u', 'URI (Uniform Resource Identifier)', 'URI (Uniform Resource Identifier)', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '883', 'w', 'Numero di controllo bibliografico', 'Numero di controllo bibliografico', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '883', '0', 'Numero del record di authority o numero standard', 'Numero del record di authority o numero standard', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '883', '8', 'Numero di collegamento e di sequenza di campi', 'Numero di collegamento e di sequenza di campi', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '884', 'a', 'Processo di conversione', 'Processo di conversione', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '884', 'g', 'Data di conversione', 'Data di conversione', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '884', 'k', 'Identificatore dei metadati di partenza', 'Identificatore dei metadati di partenza', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '884', 'q', 'Agenzia che ha effettuato la conversione', 'Agenzia che ha effettuato la conversione', 0, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '884', 'u', 'URI (Uniform Resource Identifier)', 'URI (Uniform Resource Identifier)', 1, 0, 8, NULL, NULL, NULL, 0, 0, '', '', ''),
		('', '942', 'a', 'Tipo di authority', 'Tipo di authority', 0, 1, 9, NULL, NULL, NULL, 0, 8, '', 'auth_header.authtypecode', '');


-- ******************************************************

-- *******************************************************************
-- PERSONAL NAME AUTHORITY FIELDS/SUBFIELDS. XX0
-- *******************************************************************

-- copy from default auth framework except obsoleted tags

INSERT INTO auth_tag_structure (authtypecode, tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value)
    SELECT 'PERSO_NAME', tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value
    FROM auth_tag_structure
    WHERE authtypecode = '' AND
    tagfield NOT IN ( '058','063','068','084','088','140','143','153',
                      '154','253','353','453','553','668','671','676',
                      '684','685','686','687','753','754',
                      '761','763','764','765','766','768') AND
    tagfield NOT IN ('110','111','130','148','150','151','155','162',
                     '180','181','182','185','034','640','641','642',
                     '643','644','645','646');

INSERT INTO auth_subfield_structure (authtypecode, tagfield, tagsubfield, liblibrarian, libopac, repeatable,
mandatory, tab, authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode)
    SELECT 'PERSO_NAME', tagfield, tagsubfield, liblibrarian, libopac, repeatable, mandatory, tab,
    authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode
    FROM auth_subfield_structure
    WHERE authtypecode = '' AND
    tagfield NOT IN ( '058','063','068','084','088','140','143','153',
                      '154','253','353','453','553','668','671','676',
                      '684','685','686','687','753','754',
                      '761','763','764','765','766','768') AND
    tagfield NOT IN ('110','111','130','148','150','151','155','162',
                     '180','181','182','185','034','640','641','642',
                     '643','644','645','646');

-- ******************************************************

-- *******************************************************************
-- CORPORATE NAME AUTHORITY FIELDS/SUBFIELDS. X10
-- *******************************************************************

-- copy from default auth framework except obsoleted tags

INSERT INTO auth_tag_structure (authtypecode, tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value)
    SELECT 'CORPO_NAME', tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value
    FROM auth_tag_structure
    WHERE authtypecode = '' AND
    tagfield NOT IN ( '058','063','068','084','088','140','143','153',
                      '154','253','353','453','553','668','671','676',
                      '684','685','686','687','753','754',
                      '761','763','764','765','766','768') AND
    tagfield NOT IN ( '100','111','130','148','150','151','155','162',
                      '180','181','182','185','034','640','641','642',
                      '643','644','645','646');

INSERT INTO auth_subfield_structure (authtypecode, tagfield, tagsubfield, liblibrarian, libopac, repeatable,
mandatory, tab, authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode)
    SELECT 'CORPO_NAME', tagfield, tagsubfield, liblibrarian, libopac, repeatable, mandatory, tab,
    authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode
    FROM auth_subfield_structure
    WHERE authtypecode = '' AND
    tagfield NOT IN ( '058','063','068','084','088','140','143','153',
                      '154','253','353','453','553','668','671','676',
                      '684','685','686','687','753','754',
                      '761','763','764','765','766','768') AND
    tagfield NOT IN ( '100','111','130','148','150','151','155','162',
                      '180','181','182','185','034','640','641','642',
                      '643','644','645','646');

-- ******************************************************

-- *******************************************************************
-- MEETING NAME AUTHORITY FIELDS/SUBFIELDS. X11
-- *******************************************************************

-- copy from default auth framework except obsoleted tags

INSERT INTO auth_tag_structure (authtypecode, tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value)
    SELECT 'MEETI_NAME', tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value
    FROM auth_tag_structure
    WHERE authtypecode = '' AND
    tagfield NOT IN ( '058','063','068','084','088','140','143','153',
                      '154','253','353','453','553','668','671','676',
                      '684','685','686','687','753','754',
                      '761','763','764','765','766','768') AND
    tagfield NOT IN ( '100','110','130','148','150','151','155','162',
                      '180','181','182','185','640','641','642','643',
                      '644','645','646');

INSERT INTO auth_subfield_structure (authtypecode, tagfield, tagsubfield, liblibrarian, libopac, repeatable,
mandatory, tab, authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode)
    SELECT 'MEETI_NAME', tagfield, tagsubfield, liblibrarian, libopac, repeatable, mandatory, tab,
    authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode
    FROM auth_subfield_structure
    WHERE authtypecode = '' AND
    tagfield NOT IN ( '058','063','068','084','088','140','143','153',
                      '154','253','353','453','553','668','671','676',
                      '684','685','686','687','753','754',
                      '761','763','764','765','766','768') AND
    tagfield NOT IN ( '100','110','130','148','150','151','155','162',
                      '180','181','182','185','640','641','642','643',
                      '644','645','646');

-- ******************************************************

-- *******************************************************************
-- UNIFORM TITLE AUTHORITY FIELDS/SUBFIELDS. X30
-- *******************************************************************

-- copy from default auth framework except obsoleted tags

INSERT INTO auth_tag_structure (authtypecode, tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value)
    SELECT 'UNIF_TITLE', tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value
    FROM auth_tag_structure
    WHERE authtypecode = '' AND
    tagfield NOT IN ( '058','063','068','084','088','140','143','153',
                      '154','253','353','453','553','668','671','676',
                      '684','685','686','687','753','754',
                      '761','763','764','765','766','768') AND
    tagfield NOT IN ( '100','110','111','148','150','151','155','162',
                      '180','181','182','185');

INSERT INTO auth_subfield_structure (authtypecode, tagfield, tagsubfield, liblibrarian, libopac, repeatable,
mandatory, tab, authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode)
    SELECT 'UNIF_TITLE', tagfield, tagsubfield, liblibrarian, libopac, repeatable, mandatory, tab,
    authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode
    FROM auth_subfield_structure
    WHERE authtypecode = '' AND
    tagfield NOT IN ( '058','063','068','084','088','140','143','153',
                      '154','253','353','453','553','668','671','676',
                      '684','685','686','687','753','754',
                      '761','763','764','765','766','768') AND
    tagfield NOT IN ( '100','110','111','148','150','151','155','162',
                      '180','181','182','185');

-- ******************************************************

-- *******************************************************************
-- CHRONOLOGICAL TERMS AUTHORITY FIELDS/SUBFIELDS. X48
-- *******************************************************************

-- copy from default auth framework except obsoleted tags

INSERT INTO auth_tag_structure (authtypecode, tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value)
    SELECT 'CHRON_TERM', tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value
    FROM auth_tag_structure
    WHERE authtypecode = '' AND
    tagfield NOT IN ( '058','063','068','084','088','140','143','153',
                      '154','253','353','453','553','668','671','676',
                      '684','685','686','687','753','754',
                      '761','763','764','765','766','768') AND
    tagfield NOT IN ( '100','110','111','130','150','151','155','162',
                      '180','181','185','014','020','022','031','640',
                      '641','642','643','644','645','646');

INSERT INTO auth_subfield_structure (authtypecode, tagfield, tagsubfield, liblibrarian, libopac, repeatable,
mandatory, tab, authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode)
    SELECT 'CHRON_TERM', tagfield, tagsubfield, liblibrarian, libopac, repeatable, mandatory, tab,
    authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode
    FROM auth_subfield_structure
    WHERE authtypecode = '' AND
    tagfield NOT IN ( '058','063','068','084','088','140','143','153',
                      '154','253','353','453','553','668','671','676',
                      '684','685','686','687','753','754',
                      '761','763','764','765','766','768') AND
    tagfield NOT IN ( '100','110','111','130','150','151','155','162',
                      '180','181','185','014','020','022','031','640',
                      '641','642','643','644','645','646');

-- ******************************************************

-- *******************************************************************
-- TOPICAL TERMS AUTHORITY FIELDS/SUBFIELDS. X50
-- *******************************************************************

-- copy from default auth framework except obsoleted tags

INSERT INTO auth_tag_structure (authtypecode, tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value)
    SELECT 'TOPIC_TERM', tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value
    FROM auth_tag_structure
    WHERE authtypecode = '' AND
    tagfield NOT IN ( '058','063','068','084','088','140','143','153',
                      '154','253','353','453','553','668','671','676',
                      '684','685','686','687','753','754',
                      '761','763','764','765','766','768') AND
    tagfield NOT IN ( '100','110','111','130','148','151','155','162',
                      '181','182','185','014','020','022','031','640',
                      '641','642','643','644','645','646');

INSERT INTO auth_subfield_structure (authtypecode, tagfield, tagsubfield, liblibrarian, libopac, repeatable,
mandatory, tab, authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode)
    SELECT 'TOPIC_TERM', tagfield, tagsubfield, liblibrarian, libopac, repeatable, mandatory, tab,
    authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode
    FROM auth_subfield_structure
    WHERE authtypecode = '' AND
    tagfield NOT IN ( '058','063','068','084','088','140','143','153',
                      '154','253','353','453','553','668','671','676',
                      '684','685','686','687','753','754',
                      '761','763','764','765','766','768') AND
    tagfield NOT IN ( '100','110','111','130','148','151','155','162',
                      '181','182','185','014','020','022','031','640',
                      '641','642','643','644','645','646');

-- ******************************************************

-- *******************************************************************
-- GEOGRAPHIC NAMES AUTHORITY FIELDS/SUBFIELDS. X51
-- *******************************************************************

-- copy from default auth framework except obsoleted tags

INSERT INTO auth_tag_structure (authtypecode, tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value)
    SELECT 'GEOGR_NAME', tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value
    FROM auth_tag_structure
    WHERE authtypecode = '' AND
    tagfield NOT IN ( '058','063','068','084','088','140','143','153',
                      '154','253','353','453','553','668','671','676',
                      '684','685','686','687','753','754',
                      '761','763','764','765','766','768') AND
    tagfield NOT IN ( '100','110','111','130','148','150','155','162',
                      '180','182','185','014','020','022','031','640',
                      '641','642','643','644','645','646');

INSERT INTO auth_subfield_structure (authtypecode, tagfield, tagsubfield, liblibrarian, libopac, repeatable,
mandatory, tab, authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode)
    SELECT 'GEOGR_NAME', tagfield, tagsubfield, liblibrarian, libopac, repeatable, mandatory, tab,
    authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode
    FROM auth_subfield_structure
    WHERE authtypecode = '' AND
    tagfield NOT IN ( '058','063','068','084','088','140','143','153',
                      '154','253','353','453','553','668','671','676',
                      '684','685','686','687','753','754',
                      '761','763','764','765','766','768') AND
    tagfield NOT IN ( '100','110','111','130','148','150','155','162',
                      '180','182','185','014','020','022','031','640',
                      '641','642','643','644','645','646');

-- ******************************************************

-- *******************************************************************
-- GENRE/FORM TERMS AUTHORITY FIELDS/SUBFIELDS. X55
-- *******************************************************************

-- copy from default auth framework except obsoleted tags

INSERT INTO auth_tag_structure (authtypecode, tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value)
    SELECT 'GENRE/FORM', tagfield, liblibrarian, libopac, repeatable, mandatory, authorised_value
    FROM auth_tag_structure
    WHERE authtypecode = '' AND
    tagfield NOT IN ( '058','063','068','084','088','140','143','153',
                      '154','253','353','453','553','668','671','676',
                      '684','685','686','687','753','754',
                      '761','763','764','765','766','768') AND
    tagfield NOT IN ( '100','110','111','130','148','150','151','162',
                      '180','181','182','014','020','022','031','640',
                      '641','642','643','644','645','646');

INSERT INTO auth_subfield_structure (authtypecode, tagfield, tagsubfield, liblibrarian, libopac, repeatable,
mandatory, tab, authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode)
    SELECT 'GENRE/FORM', tagfield, tagsubfield, liblibrarian, libopac, repeatable, mandatory, tab,
    authorised_value, value_builder, seealso, isurl, hidden, linkid, kohafield, frameworkcode
    FROM auth_subfield_structure
    WHERE authtypecode = '' AND
    tagfield NOT IN ( '058','063','068','084','088','140','143','153',
                      '154','253','353','453','553','668','671','676',
                      '684','685','686','687','753','754',
                      '761','763','764','765','766','768') AND
    tagfield NOT IN ( '100','110','111','130','148','150','151','162',
                      '180','181','182','014','020','022','031','640',
                      '641','642','643','644','645','646');

-- ******************************************************
